@@ -1,7 +1,7 @@
 /*
  * call center module - call queuing and distribution
  *
- * Copyright (C) 2014 OpenSIPS Solutions
+ * Copyright (C) 2014-2020 OpenSIPS Solutions
  *
  * This file is part of opensips, a free SIP server.
  *
@@ -17,10 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
- *
- * History:
- * --------
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -28,25 +25,100 @@
 
 #include "../../globals.h"
 #include "../../db/db.h"
+#include "../../timer.h"
 #include "../b2b_logic/b2b_load.h"
 #include "cc_db.h"
 
-#define CC_FETCH_ROWS     100
+
+#define CC_FLOW_TABLE_NAME "cc_flows"
+#define CC_FLOW_TABLE_VERSION  2
+#define CCF_FLOWID_COL "flowid"
+#define CCF_PRIORITY_ID_COL "priority"
+#define CCF_SKILL_COL "skill"
+#define CCF_CID_COL "prependcid"
+#define CCF_MAX_WRAPUP_COL "max_wrapup_time"
+#define CCF_DISSUADING_HANGUP_COL "dissuading_hangup"
+#define CCF_DISSUADING_ONHOLD_TH_COL "dissuading_onhold_th"
+#define CCF_DISSUADING_EWT_TH_COL "dissuading_ewt_th"
+#define CCF_DISSUADING_QSIZE_TH_COL "dissuading_qsize_th"
+#define CCF_WELCOME_COL "message_welcome"
+#define CCF_M_QUEUE_COL "message_queue"
+#define CCF_M_DISSUADING_COL "message_dissuading"
+#define CCF_M_FLOW_ID_COL "message_flow_id"
+
+#define CC_AGENT_TABLE_NAME "cc_agents"
+#define CC_AGENT_TABLE_VERSION  3
+#define CCA_AGENTID_COL "agentid"
+#define CCA_LOCATION_COL "location"
+#define CCA_MSRP_LOCATION_COL "msrp_location"
+#define CCA_MSRP_MAX_SESSIONS_COL "msrp_max_sessions"
+#define CCA_SKILLS_COL "skills"
+#define CCA_LOGSTATE_COL "logstate"
+#define CCA_WRAPUPEND_COL "wrapup_end_time"
+#define CCA_WRAPUPTIME_COL "wrapup_time"
+
+#define CC_GLOBALS_TABLE_NAME "cc_globals"
+#define CC_GLOBALS_TABLE_VERSION  1
+#define CCG_NAME_COL "name"
+#define CCG_VALUE_COL "value"
+
+#define CC_CDRS_TABLE_NAME "cc_cdrs"
+#define CCC_CALLER_COL "caller"
+#define CCC_RECV_TIME_COL "received_timestamp"
+#define CCC_WAIT_TIME_COL "wait_time"
+#define CCC_TALK_TIME_COL "talk_time"
+#define CCC_PICKUP_TIME_COL "pickup_time"
+#define CCC_FLOW_ID_COL "flow_id"
+#define CCC_AGENT_ID_COL "agent_id"
+#define CCC_CC_ID_COL "callcenter_id"
+#define CCC_TYPE_COL "call_type"
+#define CCC_REJECTED_COL "rejected"
+#define CCC_FSTATS_COL "fstats"
+#define CCC_CID_COL "cid"
+#define CCC_MEDIA_COL "media"
+
+#define CC_CALLS_TABLE_NAME "cc_calls"
+#define CC_CALLS_TABLE_VERSION  3
+#define CCQ_STATE_COL       "state"
+#define CCQ_MEDIA_COL       "media"
+#define CCQ_IGCBACK_COL     "ig_cback"
+#define CCQ_NOREJ_COL       "no_rej"
+#define CCQ_SETUP_TIME_COL  "setup_time"
+#define CCQ_ETA_COL         "eta"
+#define CCQ_LAST_START_COL  "last_start"
+#define CCQ_RECV_TIME_COL   "recv_time"
+#define CCQ_CALLER_DN_COL   "caller_dn"
+#define CCQ_CALLER_UN_COL   "caller_un"
+#define CCQ_B2BUAID_COL     "b2buaid"
+#define CCQ_FLOW_COL        "flow"
+#define CCQ_AGENT_COL       "agent"
+#define CCQ_PARAM_COL       "script_param"
+#define CCQ_QID_COL         "qid"
 
 str cc_flow_table_name			=	str_init(CC_FLOW_TABLE_NAME);
 str ccf_flowid_column			=	str_init(CCF_FLOWID_COL);
 str ccf_priority_column			=	str_init(CCF_PRIORITY_ID_COL);
 str ccf_skill_column			=	str_init(CCF_SKILL_COL);
 str ccf_cid_column				=	str_init(CCF_CID_COL);
+str ccf_max_wrapup_column		=	str_init(CCF_MAX_WRAPUP_COL);
+str ccf_dissuading_hangup_column=	str_init(CCF_DISSUADING_HANGUP_COL);
+str ccf_dissuading_onhold_th_column=str_init(CCF_DISSUADING_ONHOLD_TH_COL);
+str ccf_dissuading_ewt_th_column=	str_init(CCF_DISSUADING_EWT_TH_COL);
+str ccf_dissuading_qsize_th_column=	str_init(CCF_DISSUADING_QSIZE_TH_COL);
 str ccf_m_welcome_column		=	str_init(CCF_WELCOME_COL);
 str ccf_m_queue_column			=	str_init(CCF_M_QUEUE_COL);
+str ccf_m_dissuading_column		=	str_init(CCF_M_DISSUADING_COL);
+str ccf_m_flow_id_column		=	str_init(CCF_M_FLOW_ID_COL);
 
 str cc_agent_table_name			=	str_init(CC_AGENT_TABLE_NAME);
 str cca_agentid_column			=	str_init(CCA_AGENTID_COL);
-str cca_location_column			=	str_init(CCA_LOCATION_ID_COL);
+str cca_location_column			=	str_init(CCA_LOCATION_COL);
+str cca_msrp_location_column	=	str_init(CCA_MSRP_LOCATION_COL);
+str cca_msrp_max_sessions_column=	str_init(CCA_MSRP_MAX_SESSIONS_COL);
 str cca_skills_column			=	str_init(CCA_SKILLS_COL);
 str cca_logstate_column			=	str_init(CCA_LOGSTATE_COL);
-str cca_lastcallend_column		=	str_init(CCA_LASTCALLEND_COL);
+str cca_wrapupend_column		=	str_init(CCA_WRAPUPEND_COL);
+str cca_wrapuptime_column		=	str_init(CCA_WRAPUPTIME_COL);
 
 str cc_globals_table_name		=	str_init(CC_GLOBALS_TABLE_NAME);
 str ccg_name_column				=	str_init(CCG_NAME_COL);
@@ -65,9 +137,11 @@ str ccc_type_column				=	str_init(CCC_TYPE_COL);
 str ccc_rejected_column			=	str_init(CCC_REJECTED_COL);
 str ccc_fstats_column 			=	str_init(CCC_FSTATS_COL);
 str ccc_cid_column				=	str_init(CCC_CID_COL);
+str ccc_media_column			=	str_init(CCC_MEDIA_COL);
 
 str cc_calls_table_name			=	str_init(CC_CALLS_TABLE_NAME);
 str ccq_state_column			=	str_init(CCQ_STATE_COL);
+str ccq_media_column			=	str_init(CCQ_MEDIA_COL);
 str ccq_ig_cback_column			=	str_init(CCQ_IGCBACK_COL);
 str ccq_no_rej_column			=	str_init(CCQ_NOREJ_COL);
 str ccq_setup_time_column		=	str_init(CCQ_SETUP_TIME_COL);
@@ -79,12 +153,17 @@ str ccq_caller_un_column		=	str_init(CCQ_CALLER_UN_COL);
 str ccq_b2buaid_column			=	str_init(CCQ_B2BUAID_COL);
 str ccq_flow_column				=	str_init(CCQ_FLOW_COL);
 str ccq_agent_column			=	str_init(CCQ_AGENT_COL);
-#define CCQ_COLS_NO  12
+str ccq_param_column			=	str_init(CCQ_PARAM_COL);
+#define CCQ_COLS_NO  14
+
+#define CC_FETCH_ROWS     100
 
 static db_con_t* cc_db_handle    = 0; /* database connection handle */
 static db_con_t* cc_acc_db_handle    = 0; /* database connection handle */
+static db_con_t* cc_rt_db_handle    = 0; /* database connection handle */
 static db_func_t cc_dbf;
 static db_func_t cc_acc_dbf;
+static db_func_t cc_rt_dbf;
 extern b2bl_api_t b2b_api;
 
 
@@ -129,6 +208,20 @@ int cc_connect_acc_db(const str *acc_db_url)
 	return 0;
 }
 
+
+int cc_connect_rt_db(const str *rt_db_url)
+{
+	if (cc_rt_db_handle) {
+		LM_CRIT("BUG - db connection found already open\n");
+		return -1;
+	}
+	if ((cc_rt_db_handle = cc_rt_dbf.init(rt_db_url)) == 0)
+		return -1;
+
+	return 0;
+}
+
+
 void cc_close_db(void)
 {
 	if (cc_db_handle==NULL)
@@ -139,6 +232,16 @@ void cc_close_db(void)
 }
 
 
+void cc_close_rt_db(void)
+{
+	if (cc_rt_db_handle==NULL)
+		return;
+
+	cc_rt_dbf.close(cc_rt_db_handle);
+	cc_rt_db_handle = NULL;
+}
+
+
 int init_cc_db(const str *db_url)
 {
 	/* Find a database module */
@@ -164,6 +267,11 @@ int init_cc_db(const str *db_url)
 		return -1;
 	}
 
+	if(db_check_table_version(&cc_rt_dbf, cc_db_handle,
+	&cc_calls_table_name, CC_CALLS_TABLE_VERSION) < 0) {
+		LM_ERR("error during CALLS table version check.\n");
+		return -1;
+	}
 	return 0;
 }
 
@@ -179,12 +287,29 @@ int init_cc_acc_db(const str *acc_db_url)
 }
 
 
+int init_cc_rt_db(const str *rt_db_url)
+{
+	/* Find a database module */
+	if (db_bind_mod(rt_db_url, &cc_rt_dbf) < 0){
+		LM_ERR("Unable to bind to a database driver\n");
+		return -1;
+	}
+
+	if (cc_connect_rt_db(rt_db_url)!=0){
+		LM_ERR("unable to connect to the database\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+
 int cc_db_delete_call(struct cc_call *call)
 {
 	db_key_t qcols[1];
 	db_val_t qvals[1];
 
-	if(cc_dbf.use_table( cc_db_handle, &cc_calls_table_name) < 0)
+	if(cc_rt_dbf.use_table( cc_rt_db_handle, &cc_calls_table_name) < 0)
 	{
 		LM_ERR("SQL use table for %.*s table failed\n",
 				cc_calls_table_name.len, cc_calls_table_name.s);
@@ -196,7 +321,7 @@ int cc_db_delete_call(struct cc_call *call)
 	qvals[0].nul         = 0;
 	qvals[0].val.str_val = call->b2bua_id;
 
-	if(cc_dbf.delete(cc_db_handle, qcols, 0, qvals, 1) < 0) {
+	if(cc_rt_dbf.delete(cc_rt_db_handle, qcols, 0, qvals, 1) < 0) {
 		LM_ERR("unsuccessful sql delete operation");
 		return -1;
 	}
@@ -211,7 +336,7 @@ int cc_db_update_call(struct cc_call *call)
 	db_val_t qvals[1];
 	db_val_t uvals[5];
 
-	if(cc_dbf.use_table( cc_db_handle, &cc_calls_table_name) < 0) {
+	if(cc_rt_dbf.use_table( cc_rt_db_handle, &cc_calls_table_name) < 0) {
 		LM_ERR("SQL use table for %.*s table failed\n",
 				cc_calls_table_name.len, cc_calls_table_name.s);
 		return -1;
@@ -240,7 +365,7 @@ int cc_db_update_call(struct cc_call *call)
 	if(call->agent)
 		uvals[4].val.str_val = call->agent->id;
 
-	if( cc_dbf.update(cc_db_handle, qcols, 0, qvals,
+	if( cc_rt_dbf.update(cc_rt_db_handle, qcols, 0, qvals,
 			ucols, uvals, 1, 5)<0) 
 	{
 		LM_ERR("updating call record in database\n");
@@ -256,7 +381,7 @@ int cc_db_insert_call(struct cc_call *call)
 	db_key_t columns[CCQ_COLS_NO];
 	db_val_t vals[CCQ_COLS_NO];
 
-	if(cc_dbf.use_table( cc_db_handle, &cc_calls_table_name) < 0)
+	if(cc_rt_dbf.use_table( cc_rt_db_handle, &cc_calls_table_name) < 0)
 	{
 		LM_ERR("SQL use table for %.*s table failed\n",
 				cc_calls_table_name.len, cc_calls_table_name.s);
@@ -268,42 +393,48 @@ int cc_db_insert_call(struct cc_call *call)
 	columns[0]           = &ccq_state_column;
 	vals[0].type         = DB_INT;
 	vals[0].val.int_val  = call->state;
-	columns[1]           = &ccq_ig_cback_column;
+	columns[1]           = &ccq_media_column;
 	vals[1].type         = DB_INT;
-	vals[1].val.int_val  = call->ign_cback;
-	columns[2]           = &ccq_no_rej_column;
+	vals[1].val.int_val  = call->media;
+	columns[2]           = &ccq_ig_cback_column;
 	vals[2].type         = DB_INT;
-	vals[2].val.int_val  = call->no_rejections;
-	columns[3]           = &ccq_setup_time_column;
+	vals[2].val.int_val  = call->ign_cback;
+	columns[3]           = &ccq_no_rej_column;
 	vals[3].type         = DB_INT;
-	vals[3].val.int_val  = call->setup_time;
-	columns[4]           = &ccq_eta_column;
+	vals[3].val.int_val  = call->no_rejections;
+	columns[4]           = &ccq_setup_time_column;
 	vals[4].type         = DB_INT;
-	vals[4].val.int_val  = call->eta;
-	columns[5]           = &ccq_last_start_column;
+	vals[4].val.int_val  = call->setup_time;
+	columns[5]           = &ccq_eta_column;
 	vals[5].type         = DB_INT;
-	vals[5].val.int_val  = call->last_start;
-	columns[6]           = &ccq_recv_time_column;
+	vals[5].val.int_val  = call->eta;
+	columns[6]           = &ccq_last_start_column;
 	vals[6].type         = DB_INT;
-	vals[6].val.int_val  = call->recv_time;
-	columns[7]           = &ccq_caller_dn_column;
-	vals[7].type         = DB_STR;
-	vals[7].val.str_val  = call->caller_dn;
-	columns[8]           = &ccq_caller_un_column;
+	vals[6].val.int_val  = call->last_start;
+	columns[7]           = &ccq_recv_time_column;
+	vals[7].type         = DB_INT;
+	vals[7].val.int_val  = call->recv_time;
+	columns[8]           = &ccq_caller_dn_column;
 	vals[8].type         = DB_STR;
-	vals[8].val.str_val  = call->caller_un;
-	columns[9]          = &ccq_b2buaid_column;
-	vals[9].type        = DB_STR;
-	vals[9].val.str_val = call->b2bua_id;
-	columns[10]          = &ccq_flow_column;
+	vals[8].val.str_val  = call->caller_dn;
+	columns[9]           = &ccq_caller_un_column;
+	vals[9].type         = DB_STR;
+	vals[9].val.str_val  = call->caller_un;
+	columns[10]          = &ccq_b2buaid_column;
 	vals[10].type        = DB_STR;
-	vals[10].val.str_val = call->flow->id;
-	columns[11]          = &ccq_agent_column;
+	vals[10].val.str_val = call->b2bua_id;
+	columns[11]          = &ccq_flow_column;
 	vals[11].type        = DB_STR;
+	vals[11].val.str_val = call->flow->id;
+	columns[12]          = &ccq_agent_column;
+	vals[12].type        = DB_STR;
 	if(call->agent)
-		vals[11].val.str_val = call->agent->id;
+		vals[12].val.str_val = call->agent->id;
+	columns[13]          = &ccq_param_column;
+	vals[13].type        = DB_STR;
+	vals[13].val.str_val = call->script_param;
 
-	if (cc_dbf.insert(cc_db_handle, columns, vals, CCQ_COLS_NO) < 0) {
+	if (cc_rt_dbf.insert(cc_rt_db_handle, columns, vals, CCQ_COLS_NO) < 0) {
 		LM_ERR("inserting new record in database\n");
 		return -1;
 	}
@@ -322,25 +453,27 @@ int cc_db_restore_calls( struct cc_data *data)
 	int i;
 	struct cc_agent *agent = NULL;
 	struct cc_agent *prev;
-	str dn, un;
+	str dn, un, param;
 	str id;
 
-	cc_dbf.use_table( cc_db_handle, &cc_calls_table_name);
+	cc_rt_dbf.use_table( cc_rt_db_handle, &cc_calls_table_name);
 
 	columns[0] = &ccq_state_column;
-	columns[1] = &ccq_ig_cback_column;
-	columns[2] = &ccq_no_rej_column;
-	columns[3] = &ccq_setup_time_column;
-	columns[4] = &ccq_eta_column;
-	columns[5] = &ccq_last_start_column;
-	columns[6] = &ccq_recv_time_column;
-	columns[7] = &ccq_caller_dn_column;
-	columns[8] = &ccq_caller_un_column;
-	columns[9] = &ccq_b2buaid_column;
-	columns[10] = &ccq_flow_column;
-	columns[11] = &ccq_agent_column;
+	columns[1] = &ccq_media_column;
+	columns[2] = &ccq_ig_cback_column;
+	columns[3] = &ccq_no_rej_column;
+	columns[4] = &ccq_setup_time_column;
+	columns[5] = &ccq_eta_column;
+	columns[6] = &ccq_last_start_column;
+	columns[7] = &ccq_recv_time_column;
+	columns[8] = &ccq_caller_dn_column;
+	columns[9] = &ccq_caller_un_column;
+	columns[10] = &ccq_b2buaid_column;
+	columns[11] = &ccq_flow_column;
+	columns[12] = &ccq_agent_column;
+	columns[13] = &ccq_param_column;
 
-	if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0,
+	if ( cc_rt_dbf.query( cc_rt_db_handle, 0, 0, 0, columns, 0,
 				CCQ_COLS_NO, 0, &res)<0) {
 		LM_ERR("DB query failed\n");
 		return -1;
@@ -359,8 +492,8 @@ int cc_db_restore_calls( struct cc_data *data)
 		row = RES_ROWS(res) + i;
 
 		/* FLOW_COL */
-		check_val( ROW_VALUES(row)+10, DB_STRING, 1, 1, "flow");
-		s.s = (char*)VAL_STRING(ROW_VALUES(row)+10);
+		check_val( ROW_VALUES(row)+11, DB_STRING, 1, 1, "flow");
+		s.s = (char*)VAL_STRING(ROW_VALUES(row)+11);
 		s.len = strlen(s.s);
 		flow = get_flow_by_name(data, &s);
 		if (flow==NULL) {
@@ -370,25 +503,28 @@ int cc_db_restore_calls( struct cc_data *data)
 		LM_DBG("using call flow %p\n", flow);
 
 		/* CALLER_DN_COL */
-		check_val( ROW_VALUES(row)+7, DB_STRING, 1, 0, "caller_dn");
-		dn.s = (char*)VAL_STRING(ROW_VALUES(row)+7);
-		if(dn.s)
-			dn.len = strlen(dn.s);
+		check_val( ROW_VALUES(row)+8, DB_STRING, 1, 0, "caller_dn");
+		dn.s = (char*)VAL_STRING(ROW_VALUES(row)+8);
+		dn.len = (dn.s ? strlen(dn.s) : 0);
 		/* CALLER_UN_COL */
-		check_val( ROW_VALUES(row)+8, DB_STRING, 1, 0, "caller_un");
-		un.s = (char*)VAL_STRING(ROW_VALUES(row)+8);
-		if(un.s)
-			un.len = strlen(un.s);
+		check_val( ROW_VALUES(row)+9, DB_STRING, 1, 0, "caller_un");
+		un.s = (char*)VAL_STRING(ROW_VALUES(row)+9);
+		un.len = (un.s ? strlen(un.s) : 0);
 
-		call = new_cc_call(data, flow, &dn, &un);
+		/* SCRIPT_PARAM_COL */
+		check_val( ROW_VALUES(row)+13, DB_STRING, 1, 0, "script param");
+		param.s = (char*)VAL_STRING(ROW_VALUES(row)+13);
+		param.len = (param.s ? strlen(param.s) : 0);
+
+		call = new_cc_call(data, flow, &dn, &un, &param);
 		if (call==NULL) {
 			LM_ERR("failed to create new call\n");
 			goto error;
 		}
 
 		/* AGENT_COL */
-		check_val( ROW_VALUES(row)+11, DB_STRING, 0, 0, "agent");
-		s.s = (char*)VAL_STRING(ROW_VALUES(row)+11);
+		check_val( ROW_VALUES(row)+12, DB_STRING, 0, 0, "agent");
+		s.s = (char*)VAL_STRING(ROW_VALUES(row)+12);
 		if(s.s && strlen(s.s)) {
 			s.len = strlen(s.s);
 			/* name of the agent */
@@ -398,34 +534,41 @@ int cc_db_restore_calls( struct cc_data *data)
 				continue;
 			}
 			call->agent = agent;
-			agent->state = CC_AGENT_INCALL;
 			agent->ref_cnt++;
 		}
 
 		/* STATE_COL */
 		check_val( ROW_VALUES(row), DB_INT, 1, 0, "state");
 		call->state = VAL_INT(ROW_VALUES(row));
+		/* MEDIA_COL */
+		check_val( ROW_VALUES(row)+1, DB_INT, 1, 0, "media");
+		call->media = VAL_INT(ROW_VALUES(row)+1);
+		if (agent) {
+			agent->state = (call->media==CC_MEDIA_RTP) ?
+				CC_AGENT_INCALL : CC_AGENT_INCHAT;
+			agent->ongoing_sessions[call->media]++;
+		}
 		/* IGCBACK_COL */
-		check_val( ROW_VALUES(row)+1, DB_INT, 1, 0, "ig_cback");
-		call->ign_cback = VAL_INT(ROW_VALUES(row)+1);
+		check_val( ROW_VALUES(row)+2, DB_INT, 1, 0, "ig_cback");
+		call->ign_cback = VAL_INT(ROW_VALUES(row)+2);
 		/* NOREJ_COL */
-		check_val( ROW_VALUES(row)+2, DB_INT, 1, 0, "no_rej");
-		call->no_rejections = VAL_INT(ROW_VALUES(row)+2);
+		check_val( ROW_VALUES(row)+3, DB_INT, 1, 0, "no_rej");
+		call->no_rejections = VAL_INT(ROW_VALUES(row)+3);
 		/* SETUP_TIME_COL */
-		check_val( ROW_VALUES(row)+3, DB_INT, 1, 0, "setup_time");
-		call->setup_time = VAL_INT(ROW_VALUES(row)+3);
+		check_val( ROW_VALUES(row)+4, DB_INT, 1, 0, "setup_time");
+		call->setup_time = VAL_INT(ROW_VALUES(row)+4);
 		/* ETA_COL  */
-		check_val( ROW_VALUES(row)+4, DB_INT, 1, 0, "eta");
-		call->eta = VAL_INT(ROW_VALUES(row)+4);
+		check_val( ROW_VALUES(row)+5, DB_INT, 1, 0, "eta");
+		call->eta = VAL_INT(ROW_VALUES(row)+5);
 		/* LAST_START_COL */
-		check_val( ROW_VALUES(row)+5, DB_INT, 1, 0, "last_start");
-		call->last_start = VAL_INT(ROW_VALUES(row)+5);
+		check_val( ROW_VALUES(row)+6, DB_INT, 1, 0, "last_start");
+		call->last_start = VAL_INT(ROW_VALUES(row)+6);
 		/* RECV_TIME_COL */
-		check_val( ROW_VALUES(row)+6, DB_INT, 1, 0, "recv_time");
-		call->recv_time = VAL_INT(ROW_VALUES(row)+6);
+		check_val( ROW_VALUES(row)+7, DB_INT, 1, 0, "recv_time");
+		call->recv_time = VAL_INT(ROW_VALUES(row)+7);
 		/* B2BUAID_COL */
-		check_val( ROW_VALUES(row)+9, DB_STRING, 1, 1, "b2buaid");
-		id.s = (char*)VAL_STRING(ROW_VALUES(row)+9);
+		check_val( ROW_VALUES(row)+8, DB_STRING, 1, 1, "b2buaid");
+		id.s = (char*)VAL_STRING(ROW_VALUES(row)+8);
 		if(id.s) {
 			id.len = strlen(id.s);
 			call->b2bua_id.len = id.len;
@@ -463,26 +606,33 @@ error:
 
 int cc_load_db_data( struct cc_data *data)
 {
-	db_key_t columns[6];
+	db_key_t columns[13];
 	db_res_t* res;
 	db_row_t* row;
-	int i, j, n;
+	int i, j;
 	str id,skill,cid;
-	str location;
-	unsigned int priority, logstate, last_call_end;
+	unsigned int priority, logstate, wrapup, wrapup_end_time;
+	unsigned int diss_hangup, diss_ewt_th, diss_qsize_th, diss_onhold_th;
 	str messages[MAX_AUDIO];
 
 	cc_dbf.use_table( cc_db_handle, &cc_flow_table_name);
 
-	columns[0] = &ccf_flowid_column;
-	columns[1] = &ccf_priority_column;
-	columns[2] = &ccf_skill_column;
-	columns[3] = &ccf_cid_column;
-	columns[4] = &ccf_m_welcome_column;
-	columns[5] = &ccf_m_queue_column;
+	columns[0]  = &ccf_flowid_column;
+	columns[1]  = &ccf_priority_column;
+	columns[2]  = &ccf_skill_column;
+	columns[3]  = &ccf_cid_column;
+	columns[4]  = &ccf_max_wrapup_column;
+	columns[5]  = &ccf_dissuading_hangup_column;
+	columns[6]  = &ccf_dissuading_ewt_th_column;
+	columns[7]  = &ccf_dissuading_qsize_th_column;
+	columns[8]  = &ccf_dissuading_onhold_th_column;
+	columns[9]  = &ccf_m_welcome_column;
+	columns[10] = &ccf_m_queue_column;
+	columns[11] = &ccf_m_dissuading_column;
+	columns[12] = &ccf_m_flow_id_column;
 
 	if (0/*DB_CAPABILITY(cc_dbf, DB_CAP_FETCH))*/) {
-		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 6, 0, 0 ) < 0) {
+		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 13, 0, 0 ) < 0) {
 			LM_ERR("DB query failed\n");
 			return -1;
 		}
@@ -491,21 +641,14 @@ int cc_load_db_data( struct cc_data *data)
 			return -1;
 		}
 	} else {
-		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 6, 0, &res)<0) {
+		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 13, 0, &res)<0) {
 			LM_ERR("DB query failed\n");
 			return -1;
 		}
 	}
 
-	if (RES_ROW_N(res) == 0) {
-		LM_WARN("table \"%.*s\" empty\n", cc_flow_table_name.len,
-			cc_flow_table_name.s );
-		return -1;
-	}
-
 	LM_DBG("%d records found in %.*s\n",
 		RES_ROW_N(res), cc_flow_table_name.len,cc_flow_table_name.s );
-	n = 0;
 
 	do {
 		for(i=0; i < RES_ROW_N(res); i++) {
@@ -531,13 +674,32 @@ int cc_load_db_data( struct cc_data *data)
 					cid.s = NULL; cid.len = 0;
 				}
 			}
+			/* MAX_WRAPUP_TIME column */
+			check_val( ROW_VALUES(row)+4, DB_INT, 1, 0, "max_wrapup_time");
+			wrapup = VAL_INT(ROW_VALUES(row)+4);
+			/* DISSUADING_HANGUP column */
+			check_val( ROW_VALUES(row)+5, DB_INT, 1, 0, "dissuading hangup");
+			diss_hangup = VAL_INT(ROW_VALUES(row)+5);
+			/* DISSUADING_EWT_TH column */
+			check_val( ROW_VALUES(row)+6, DB_INT, 1, 0,
+				"dissuading EWT threshold");
+			diss_ewt_th = VAL_INT(ROW_VALUES(row)+6);
+			/* DISSUADING_QSIZE_TH column */
+			check_val( ROW_VALUES(row)+7, DB_INT, 1, 0,
+				"dissuading queue size threshold");
+			diss_qsize_th = VAL_INT(ROW_VALUES(row)+7);
+			/* DISSUADING_ONHOLD_TH column */
+			check_val( ROW_VALUES(row)+8, DB_INT, 1, 0,
+				"dissuading onhold threshold");
+			diss_onhold_th = VAL_INT(ROW_VALUES(row)+8);
+
 			for( j=0 ; j<MAX_AUDIO ; j++ ) {
 				/* MESSAGE_XXXX column */
-				check_val( ROW_VALUES(row)+4+j, DB_STRING, 0, 0, "message");
-				if (VAL_NULL(ROW_VALUES(row)+4+j)) {
+				check_val( ROW_VALUES(row)+9+j, DB_STRING, 0, 0, "message");
+				if (VAL_NULL(ROW_VALUES(row)+9+j)) {
 					messages[j].s = NULL; messages[j].len = 0;
 				} else {
-					messages[j].s = (char*)VAL_STRING(ROW_VALUES(row)+4+j);
+					messages[j].s = (char*)VAL_STRING(ROW_VALUES(row)+9+j);
 					if (messages[j].s==NULL ||
 					(messages[j].len=strlen(messages[j].s))==0 ) {
 						messages[j].s = NULL; messages[j].len = 0;
@@ -545,13 +707,21 @@ int cc_load_db_data( struct cc_data *data)
 				}
 			}
 
+			/* queue audio cannot be null */
+			if ( messages[1].s == NULL) {
+				LM_ERR("queue audio cannot be null in flow %.*s -> "
+					"skipping\n", id.len,id.s);
+				continue;
+			}
+
 			/* add flow */
-			if (add_cc_flow(data,&id,priority,&skill,&cid,messages)<0) {
+			if (add_cc_flow(data, &id, priority, &skill, &cid, wrapup,
+			diss_hangup, diss_ewt_th, diss_qsize_th, diss_onhold_th,
+			messages)<0) {
 				LM_ERR("failed to add flow %.*s -> skipping\n",
 					id.len,id.s);
 				continue;
 			}
-			n++;
 		}
 		if (DB_CAPABILITY( cc_dbf, DB_CAP_FETCH)) {
 			if(cc_dbf.fetch_result(cc_db_handle, &res, CC_FETCH_ROWS)<0) {
@@ -571,12 +741,15 @@ int cc_load_db_data( struct cc_data *data)
 
 	columns[0] = &cca_agentid_column;
 	columns[1] = &cca_location_column;
-	columns[2] = &cca_skills_column;
-	columns[3] = &cca_logstate_column;
-	columns[4] = &cca_lastcallend_column;
+	columns[2] = &cca_msrp_location_column;
+	columns[3] = &cca_msrp_max_sessions_column;
+	columns[4] = &cca_logstate_column;
+	columns[5] = &cca_skills_column;
+	columns[6] = &cca_wrapupend_column;
+	columns[7] = &cca_wrapuptime_column;
 
 	if (0/*DB_CAPABILITY(cc_dbf, DB_CAP_FETCH))*/) {
-		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 5, 0, 0 ) < 0) {
+		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 8, 0, 0 ) < 0) {
 			LM_ERR("DB query failed\n");
 			return -1;
 		}
@@ -585,50 +758,65 @@ int cc_load_db_data( struct cc_data *data)
 			return -1;
 		}
 	} else {
-		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 5, 0, &res)<0) {
+		if ( cc_dbf.query( cc_db_handle, 0, 0, 0, columns, 0, 8, 0, &res)<0) {
 			LM_ERR("DB query failed\n");
 			return -1;
 		}
 	}
 
-	if (RES_ROW_N(res) == 0) {
-		LM_WARN("table \"%.*s\" empty\n", cc_agent_table_name.len,
-			cc_agent_table_name.s );
-		return -1;
-	}
-
 	LM_DBG("%d records found in %.*s\n",
 		RES_ROW_N(res), cc_agent_table_name.len,cc_agent_table_name.s );
-	n = 0;
 
 	do {
+		struct media_info media[CC_MEDIA_NO];
 		for(i=0; i < RES_ROW_N(res); i++) {
 			row = RES_ROWS(res) + i;
+			memset( media, 0, CC_MEDIA_NO*sizeof(struct media_info) );
 			/* agentID column */
 			check_val( ROW_VALUES(row), DB_STRING, 1, 1, "agentid");
 			id.s = (char*)VAL_STRING(ROW_VALUES(row));
 			id.len = strlen(id.s);
 			/* LOCATION column */
-			check_val( ROW_VALUES(row)+1, DB_STRING, 1, 1, "location");
-			location.s = (char*)VAL_STRING(ROW_VALUES(row)+1);
-			location.len = strlen(location.s);
-			/* SKILLS column */
-			check_val( ROW_VALUES(row)+2, DB_STRING, 1, 1, "skills");
-			skill.s = (char*)VAL_STRING(ROW_VALUES(row)+2);
-			skill.len = strlen(skill.s);
+			check_val( ROW_VALUES(row)+1, DB_STRING, 0, 1, "location");
+			if ( !VAL_NULL(ROW_VALUES(row)+1) ) {
+				media[CC_MEDIA_RTP].location.s =
+					(char*)VAL_STRING(ROW_VALUES(row)+1);
+				media[CC_MEDIA_RTP].location.len =
+					strlen(media[CC_MEDIA_RTP].location.s);
+				media[CC_MEDIA_RTP].sessions = 1;
+			}
+			/* MSRP LOCATION column */
+			check_val( ROW_VALUES(row)+2, DB_STRING, 0, 1, "msrp_location");
+			if ( !VAL_NULL(ROW_VALUES(row)+2) ) {
+				media[CC_MEDIA_MSRP].location.s =
+					(char*)VAL_STRING(ROW_VALUES(row)+2);
+				media[CC_MEDIA_MSRP].location.len =
+					strlen(media[CC_MEDIA_MSRP].location.s);
+				/* MAX SESSIONS column */
+				check_val( ROW_VALUES(row)+3, DB_INT, 1, 0,
+					"msrp_max_sessions");
+				media[CC_MEDIA_MSRP].sessions =  VAL_INT(ROW_VALUES(row)+3);
+			}
 			/* LOGSTATE column */
-			check_val( ROW_VALUES(row)+3, DB_INT, 1, 0, "logstate");
-			logstate = VAL_INT(ROW_VALUES(row)+3);
-			/* LAST_CALL_END column */
-			last_call_end = VAL_INT(ROW_VALUES(row)+4);
+			check_val( ROW_VALUES(row)+4, DB_INT, 1, 0, "logstate");
+			logstate = VAL_INT(ROW_VALUES(row)+4);
+			/* SKILLS column */
+			check_val( ROW_VALUES(row)+5, DB_STRING, 1, 1, "skills");
+			skill.s = (char*)VAL_STRING(ROW_VALUES(row)+5);
+			skill.len = strlen(skill.s);
+			/* WRAPUP_END_TIME column */
+			wrapup_end_time = VAL_INT(ROW_VALUES(row)+6);
+			/* WRAPUP_TIME column */
+			check_val( ROW_VALUES(row)+7, DB_INT, 1, 0, "wrapup time");
+			wrapup = VAL_INT(ROW_VALUES(row)+7);
 
 			/* add agent */
-			if (add_cc_agent(data,&id,&location,&skill,logstate,last_call_end)<0){
+			if (add_cc_agent( data, &id, media, &skill, logstate, wrapup,
+			wrapup_end_time)<0){
 				LM_ERR("failed to add agent %.*s -> skipping\n",
 					id.len,id.s);
 				continue;
 			}
-			n++;
 		}
 		if (DB_CAPABILITY( cc_dbf, DB_CAP_FETCH)) {
 			if(cc_dbf.fetch_result(cc_db_handle, &res, CC_FETCH_ROWS)<0) {
@@ -688,10 +876,11 @@ int prepare_cdr(struct cc_call *call, str *un, str *fid , str *aid)
 }
 
 
-int cc_write_cdr( str *un, str *fid, str *aid, int type, int rt, int wt, int tt, int pt, int rej, int fst, int cid)
+int cc_write_cdr( str *un, str *fid, str *aid, int type, int rt, int wt,
+		int tt, int pt, int rej, int fst, int cid, media_type media)
 {
-	db_key_t columns[11];
-	db_val_t vals[11];
+	db_key_t columns[12];
+	db_val_t vals[12];
 	static db_ps_t my_ps = NULL;
 
 	cc_acc_dbf.use_table( cc_acc_db_handle, &cc_cdrs_table_name);
@@ -707,6 +896,7 @@ int cc_write_cdr( str *un, str *fid, str *aid, int type, int rt, int wt, int tt,
 	columns[8] = &ccc_rejected_column;
 	columns[9]= &ccc_fstats_column;
 	columns[10]= &ccc_cid_column;
+	columns[11]= &ccc_media_column;
 
 	/* caller */
 	vals[0].nul = 0;
@@ -767,9 +957,14 @@ int cc_write_cdr( str *un, str *fid, str *aid, int type, int rt, int wt, int tt,
 	vals[10].type = DB_INT;
 	vals[10].val.int_val = cid;
 
-	CON_PS_REFERENCE(cc_acc_db_handle) = &my_ps;
+	/* cid */
+	vals[11].nul = 0;
+	vals[11].type = DB_INT;
+	vals[11].val.int_val = (media==CC_MEDIA_RTP)? 1 :
+		((media==CC_MEDIA_MSRP)? 2 : 0) ;
 
-	if (cc_acc_dbf.insert( cc_acc_db_handle, columns, vals, 11) < 0) {
+	CON_SET_CURR_PS(cc_acc_db_handle, &my_ps);
+	if (cc_acc_dbf.insert( cc_acc_db_handle, columns, vals, 12) < 0) {
 		LM_ERR("CDR insert failed\n");
 		return -1;
 	}
@@ -778,13 +973,13 @@ int cc_write_cdr( str *un, str *fid, str *aid, int type, int rt, int wt, int tt,
 }
 
 
-void cc_db_update_agent_end_call(struct cc_agent* agent)
+void cc_db_update_agent_wrapup_end(struct cc_agent* agent)
 {
 	db_key_t columns[2];
 	db_val_t vals[2];
 
 	columns[0] = &cca_agentid_column;
-	columns[1] = &cca_lastcallend_column;
+	columns[1] = &cca_wrapupend_column;
 
 	vals[0].nul = 0;
 	vals[0].type = DB_STR;
@@ -792,11 +987,12 @@ void cc_db_update_agent_end_call(struct cc_agent* agent)
 
 	vals[1].nul = 0;
 	vals[1].type = DB_INT;
-	vals[1].val.int_val = (int)time(NULL);
+	/* translate from internal time to timestamp */
+	vals[1].val.int_val = (int)time(NULL)-get_ticks()+agent->wrapup_end_time;
 
 	cc_dbf.use_table( cc_db_handle, &cc_agent_table_name);
 
-	if (cc_dbf.update( cc_db_handle, columns, 0, vals, columns+1, vals+1, 1, 1) < 0) {
+	if (cc_dbf.update(cc_db_handle,columns,0,vals,columns+1,vals+1,1,1)<0) {
 		LM_ERR("Agent update failed\n");
 	}
 }
