@@ -36,6 +36,7 @@ typedef struct _http_conn
 {
 
 	CURL * handle;
+	struct curl_slist *headers;
 	str  start;
 	int last_id;
 } http_conn_t;
@@ -292,7 +293,8 @@ size_t receive(void *buffer, size_t size, size_t nmemb, void *userp)
 	temp.s = (char*)buffer;
 	temp.len = size * nmemb;
 
-	append_str(buff,temp);
+	if (append_str(buff,temp))
+		LM_ERR("cannot append buffer!\n");
 
 
 	return temp.len;
@@ -405,7 +407,7 @@ int put_value_in_result(  char * start, int len , db_res_t * res ,
 
 	db_val_t * row;
 
-	LM_DBG("Found value: %.*s\n",len,start);
+	LM_DBG("Found type and value: [%d][%.*s]\n",res->col.types[cur_col],len,start);
 
 	row = res->rows[cur_line].values;
 	row[cur_col].type = res->col.types[cur_col];
@@ -512,8 +514,10 @@ int form_result(var_str buff, db_res_t** r)
 				if( cur_line == -1 )
 					col_count = cur_col;
 				else
-					if(cur_col != col_count)
+					if(cur_col != col_count) {
+						LM_ERR("cur_col=[%d] != col_count=[%d]\n", cur_col, col_count);
 						goto error_before;
+					}
 
 				delim_count++;
 				cur_line++;
@@ -539,15 +543,19 @@ int form_result(var_str buff, db_res_t** r)
 	line_count = cur_line;
 
 
-	if( col_count == 0 || line_count == 0 )
+	if( col_count == 0 || line_count == 0 ) {
+		LM_ERR("col_count=[%d] or line_count=[%d] is zero\n", col_count, line_count);
 		goto error_before;
+	}
 
 
 
 	/* validate input */
 
-	if( delim_count != (line_count+1)*col_count)
+	if( delim_count != (line_count+1)*col_count) {
+		LM_ERR("delim_count=[%d] != (line_count=[%d]+1)*col_count=[%d]\n", delim_count, line_count, col_count);
 		goto error_before;
+	}
 
 
 
@@ -796,7 +804,7 @@ int do_http_op (  const db_con_t* h, const db_key_t* k, const db_op_t* op,
 
 	LM_DBG("Sent:%s \n",q.s);
 
-	curl_easy_setopt(conn->handle, CURLOPT_HTTPGET, 1);
+	curl_easy_setopt(conn->handle, CURLOPT_HTTPGET, 1L);
 	curl_easy_setopt(conn->handle, CURLOPT_URL, q.s);
 
 
@@ -811,7 +819,7 @@ int do_http_op (  const db_con_t* h, const db_key_t* k, const db_op_t* op,
 	}
 
 
-	curl_easy_setopt(conn->handle, CURLOPT_FAILONERROR,1);
+	curl_easy_setopt(conn->handle, CURLOPT_FAILONERROR,1L);
 	ret = curl_easy_perform(conn->handle);
 
 
@@ -887,7 +895,7 @@ str value_to_string(const db_val_t * v)
 			rez = v ->val.str_val;
 			break;
 		case(DB_DATETIME):
-			sprintf(buff,"%s",ctime(&v->val.time_val));
+			ctime_r(&v->val.time_val, buff);
 			rez.s = buff;
 			rez.len = strlen(rez.s);
 			break;
@@ -1037,8 +1045,17 @@ db_con_t* db_http_init(const str* url)
 	}
 
 	curl->handle = curl_easy_init();
-	curl_easy_setopt(curl->handle,CURLOPT_SSL_VERIFYPEER,0);
-	curl_easy_setopt(curl->handle,CURLOPT_SSL_VERIFYHOST,0);
+
+	curl->headers = NULL;
+
+	//Disable Expect: 100-continue behavior
+	if (disable_expect) {
+		curl->headers = curl_slist_append(curl->headers, "Expect:");
+		curl_easy_setopt(curl->handle,CURLOPT_HTTPHEADER,curl->headers);
+	}
+
+	curl_easy_setopt(curl->handle,CURLOPT_SSL_VERIFYPEER,0L);
+	curl_easy_setopt(curl->handle,CURLOPT_SSL_VERIFYHOST,0L);
 
 	curl_easy_setopt(curl->handle,CURLOPT_USERPWD,user_pass);
 	curl_easy_setopt(curl->handle,CURLOPT_HTTPAUTH,CURLAUTH_ANY);
@@ -1046,7 +1063,7 @@ db_con_t* db_http_init(const str* url)
 	curl_easy_setopt(curl->handle,CURLOPT_ERRORBUFFER,error_buffer);
 #if LIBCURL_VERSION_NUM >= 0x071002
 	LM_DBG("timeout set to %d", db_http_timeout);
-	curl_easy_setopt(curl->handle,CURLOPT_TIMEOUT_MS,db_http_timeout);
+	curl_easy_setopt(curl->handle,CURLOPT_TIMEOUT_MS,(long)db_http_timeout);
 #endif
 
 	ret = snprintf(path, DB_HTTP_BUFF_SIZE, "http");
@@ -1092,6 +1109,9 @@ db_con_t* db_http_init(const str* url)
 	{
 		pkg_free(path);
 		curl_easy_cleanup(curl->handle);
+		if (curl->headers != NULL) {
+			curl_slist_free_all(curl->headers);
+		}
 		pkg_free(curl);
 
 		LM_ERR("Out of memory\n");
@@ -1120,6 +1140,9 @@ error:
 		pkg_free(path);
 	if (curl) {
 		curl_easy_cleanup(curl->handle);
+		if (curl->headers != NULL) {
+			curl_slist_free_all(curl->headers);
+		}
 		pkg_free(curl);
 	}
 	LM_CRIT("Initialization error\n");
@@ -1132,6 +1155,7 @@ void db_http_close(db_con_t* _h)
 
 	http_conn_t* conn = (http_conn_t*) _h->tail;
 	curl_easy_cleanup(conn->handle);
+	curl_slist_free_all(conn->headers);
 	pkg_free(_h);
 }
 
@@ -1316,6 +1340,5 @@ int db_insert_update(const db_con_t* _h, const db_key_t* _k, const db_val_t* _v,
  */
 int db_http_use_table(db_con_t* _h, const str* _t)
 {
-	_h->table = _t;
-	return 0;
+	return db_use_table(_h, _t);
 }
