@@ -712,8 +712,6 @@ int dp_apply_policy(struct sip_msg* _msg, char* _s1, char* _s2) {
 	int len, didsomething;
 	char *at; /* pointer to current location inside duri */
 
-	str host;
-	int port, proto;
 	struct socket_info* si;
 
 	if (route_type != REQUEST_ROUTE) {
@@ -734,12 +732,7 @@ int dp_apply_policy(struct sip_msg* _msg, char* _s1, char* _s2) {
 			return -1;
 		}
 		LM_DBG("send_socket_avp found = '%.*s'\n", val.s.len, ZSW(val.s.s));
-		/* parse phostport */
-		if (parse_phostport(val.s.s, val.s.len, &(host.s), &(host.len), &port, &proto)) {
-			LM_ERR("could not parse send_socket, return with error ...\n");
-			return -1;
-		}
-		si = grep_sock_info( &host, (unsigned short) port, (unsigned short) proto);
+		si = parse_sock_info(&val.s);
 		if (si) {
 			_msg->force_send_socket = si;
 		} else {
@@ -911,7 +904,10 @@ int dp_apply_policy(struct sip_msg* _msg, char* _s1, char* _s2) {
 	duri_str.s = (char *)&(duri[0]);
 	duri_str.len = at - duri_str.s;
 	LM_DBG("new DURI is '%.*s'\n",duri_str.len, ZSW(duri_str.s));
-	set_dst_uri(_msg, &duri_str);
+	if (set_dst_uri(_msg, &duri_str) < 0) {
+		LM_ERR("Cannot set destination uri %.*s!\n", duri_str.len, ZSW(duri_str.s));
+		return -1;
+	}
 
 	return 1;
 }
