@@ -41,7 +41,6 @@ const char *LOCATION_TAG_BEGIN = "<location-key>";
 const char *LOCATION_TAG_END = "</location-key>";
 const char *NEW_LINE = "\n";
 
-const char *CONTENT_TYPE_PIDF = "Content-Type: application/pidf+xml";
 const char *PRESENCE_START = "<presence";
 const char *PRESENCE_END = "/presence>";
 
@@ -52,6 +51,9 @@ const char *EVENT_TYPE = "dialog";
 const char *CALLID_PARAM = "call-id=";
 const char *FROMTAG_PARAM = ";from-tag=";
 
+
+#define MIME_PIDF                "application/pidf+xml"
+#define MIME_PIDF_LEN            (sizeof(MIME_PIDF)-1)
 #define PAI_SUFFIX               ";user=phone;CBN="
 #define PAI_SUFFIX_LEN           (sizeof(PAI_SUFFIX)-1)
 #define PAI_SUFFIX_II            ";user=phone>\n"
@@ -72,17 +74,16 @@ const char *FROMTAG_PARAM = ";from-tag=";
 
 
 struct lump *l;
-struct rr_binds rr_api;
 
 /* verify if the INVITE has the header Geolocation-Routing with the value "yes"
 */
 int check_geolocation_header(struct sip_msg *msg) {
-	LM_DBG(" --- check_geolocation_header\n\n");
+	LM_DBG(" --- check_geolocation_header\n");
 	if (parse_headers(msg, HDR_OTHER_F, 0) == -1) {
 		LM_ERR("NO HEADER header\n");
 		return 0;
 	}
-	LM_DBG(" --- check_geolocation_header --- OK\n\n");
+	LM_DBG(" --- check_geolocation_header --- OK\n");
 	struct hdr_field* atual = msg->headers;
 	while (atual != NULL) {
 		char* name = pkg_malloc(sizeof (char) * atual->name.len);
@@ -118,7 +119,7 @@ int get_subscription_state_header(struct sip_msg *msg, char** subs_state, char**
 	str replacement_state;
 	str replacement_expires;
 
-	LM_DBG(" --- get_subscription_state_header\n\n");
+	LM_DBG(" --- get_subscription_state_header\n");
 	if (parse_headers(msg, HDR_OTHER_F, 0) == -1) {
 		LM_ERR("NO HEADER header\n");
 		return 0;
@@ -126,7 +127,7 @@ int get_subscription_state_header(struct sip_msg *msg, char** subs_state, char**
 
 	struct hdr_field* atual = msg->headers;
 	while (atual != NULL) {
-		LM_DBG(" --- HEADERS: %.*s\n\n",  atual->name.len, atual->name.s );
+		LM_DBG(" --- HEADERS: %.*s\n",  atual->name.len, atual->name.s );
 		if ( strncmp(atual->name.s , SUBSCRIPTION_STATE, atual->name.len) == 0){
 
 			body = pkg_malloc(sizeof (char)*atual->body.len + 1);
@@ -143,7 +144,7 @@ int get_subscription_state_header(struct sip_msg *msg, char** subs_state, char**
 				return 1;
 			}
 
-			LM_DBG(" --- Subscription_state body: %.*s\n\n", atual->body.len, atual->body.s);
+			LM_DBG(" --- Subscription_state body: %.*s\n", atual->body.len, atual->body.s);
 			state_aux = pkg_malloc(sizeof (char)*MAXNUMBERLEN);
 			if (state_aux == NULL) {
 				LM_ERR("no more pkg memory\n");
@@ -159,7 +160,7 @@ int get_subscription_state_header(struct sip_msg *msg, char** subs_state, char**
 			replacement_state.len = strlen(replacement_state.s);
 
 			if (reg_replace(pattern_state.s, replacement_state.s, atual->body.s, &pt_state) == 1) {
-				LM_DBG(" --- REPLACE OK\n\n");
+				LM_DBG(" --- REPLACE OK\n");
 				*subs_state = state_aux;
 
 				expires_aux = pkg_malloc(sizeof (char)*MAXNUMBERLEN);
@@ -180,7 +181,7 @@ int get_subscription_state_header(struct sip_msg *msg, char** subs_state, char**
 
 			}
 
-			LM_DBG(" --- REPLACE NOK\n\n");
+			LM_DBG(" --- REPLACE NOK\n");
 			return 0;
 
 		}
@@ -228,7 +229,7 @@ int get_event_header(struct sip_msg *msg, char** subs_callid, char** from_tag) {
 	str replacement_callid;
 	str replacement_ftag;
 
-	LM_DBG(" --- get_event_header\n\n");
+	LM_DBG(" --- get_event_header\n");
 	if (parse_headers(msg, HDR_OTHER_F, 0) == -1) {
 		LM_ERR("NO HEADER header\n");
 		return 0;
@@ -236,7 +237,7 @@ int get_event_header(struct sip_msg *msg, char** subs_callid, char** from_tag) {
 
 	if (msg->event != NULL && msg->event->body.len > 0){
 
-		LM_DBG(" --- Event body: %.*s\n\n",msg->event->body.len, msg->event->body.s);
+		LM_DBG(" --- Event body: %.*s\n",msg->event->body.len, msg->event->body.s);
 		callid_aux = pkg_malloc(sizeof (char)*MAXNUMBERLEN);
 		if (callid_aux == NULL) {
 			LM_ERR("no more pkg memory\n");
@@ -252,7 +253,7 @@ int get_event_header(struct sip_msg *msg, char** subs_callid, char** from_tag) {
 		replacement_callid.len = strlen(replacement_callid.s);
 
 		if (reg_replace(pattern_callid.s, replacement_callid.s, msg->event->body.s, &pt_callid) == 1) {
-			LM_DBG(" --- REPLACE OK\n\n");
+			LM_DBG(" --- REPLACE OK\n");
 			*subs_callid = callid_aux;
 
 			ftag_aux = pkg_malloc(sizeof (char)*MAXNUMBERLEN);
@@ -267,7 +268,7 @@ int get_event_header(struct sip_msg *msg, char** subs_callid, char** from_tag) {
 			replacement_ftag.s = "\\2";
 			replacement_ftag.len = strlen(replacement_ftag.s);
 			if (reg_replace(pattern_callid.s, replacement_ftag.s, msg->event->body.s, &pt_ftag) == 1) {
-				LM_DBG(" --- REPLACE OK II\n\n");
+				LM_DBG(" --- REPLACE OK II\n");
 				*from_tag = ftag_aux;
 				return 1;
 			}
@@ -275,7 +276,7 @@ int get_event_header(struct sip_msg *msg, char** subs_callid, char** from_tag) {
 
 		}
 		pkg_free(callid_aux);
-		LM_DBG(" --- REPLACE NOK\n\n");
+		LM_DBG(" --- REPLACE NOK\n");
 
 	}
 
@@ -295,13 +296,13 @@ int get_geolocation_header(struct sip_msg *msg, char** locationHeader) {
 	char* name;
 	char* body;
 
-	LM_DBG(" --- get_geolocation_header\n\n");
+	LM_DBG(" --- get_geolocation_header\n");
 	if (parse_headers(msg, HDR_OTHER_F, 0) == -1) {
 		LM_ERR("NO HEADER header\n");
 		return -1;
 	}
 
-	LM_DBG(" --- get_geolocation_header --- INICIO %s \n\n", locationTotalHeader);
+	LM_DBG(" --- get_geolocation_header --- INICIO %s \n", locationTotalHeader);
 	struct hdr_field* atual = msg->headers;
 	while (atual != NULL) {
 
@@ -350,13 +351,13 @@ int get_geolocation_header(struct sip_msg *msg, char** locationHeader) {
 				pkg_free(locationTotalHeader);
 
 			locationTotalHeader = aux;
-			LM_DBG(" --- get_geolocation_header ATUAL %s \n\n", locationTotalHeader);
+			LM_DBG(" --- get_geolocation_header ATUAL %s \n", locationTotalHeader);
 		}
 		atual = atual->next;
 	}
 
 	*locationHeader = locationTotalHeader;
-	LM_DBG(" --- get_geolocation_header FINAL %s \n\n", *locationHeader);
+	LM_DBG(" --- get_geolocation_header FINAL %s \n", *locationHeader);
 	return 1;
 }
 
@@ -470,7 +471,7 @@ int found_CBN(struct sip_msg *msg, char** cbn_aux) {
 			pkg_free(header_aux);
 
 		} else {
-			LM_ERR("****** FROM: ERRO");
+			LM_ERR("****** FROM: ERRO\n");
 			return -1;
 		}
 
@@ -487,7 +488,7 @@ error:
 */
 int check_event_header(struct sip_msg *msg) {
 
-	LM_DBG(" --- get_event_header\n\n");
+	LM_DBG(" --- get_event_header\n");
 	if (parse_headers(msg, HDR_OTHER_F, 0) == -1) {
 		LM_ERR("NO HEADER header\n");
 		return 0;
@@ -511,49 +512,29 @@ int check_event_header(struct sip_msg *msg) {
 int get_ip_socket(struct sip_msg *msg, char** saddr){
 
 	char *socket;
-	struct socket_info** list;
 	struct socket_info* si;
 
-	list = get_sock_info_list(msg->rcv.proto);
-	if (list == NULL) {
-		LM_ERR("ERROR in SOCKET\n");
+	si = msg->rcv.bind_address;
+
+	socket = pkg_malloc(si->address_str.len + si->port_no_str.len + 3);
+	if (socket == NULL) {
+		LM_ERR("no more pkg memory\n");
 		return -1;
 	}
 
-	si = *list;
-	*saddr = NULL;
-
-	while (si) {
-		if (si->port_no == msg->rcv.dst_port) {
-			socket = pkg_malloc(si->address_str.len + si->port_no_str.len + 3);
-			if (socket == NULL) {
-				LM_ERR("no more pkg memory\n");
-				return -1;
-			}
-
-			*saddr = socket;
-			*socket = '@';
-			socket++;
-			memcpy(socket, si->address_str.s, si->address_str.len);
-			socket = socket + si->address_str.len;
-			*socket = ':';
-			socket++;
-			memcpy(socket, si->port_no_str.s, si->port_no_str.len);
-			socket = socket + si->port_no_str.len;
-			*socket = 0;
-
-			LM_DBG(" --- SERVER = %s \n \n", *saddr);
-			break;
-		}
-		si = si->next;
-	}
-	if (*saddr == NULL) {
-		LM_ERR("failed in found ip listen\n");
-		return -1;
-	}
+	*saddr = socket;
+	*socket = '@';
+	socket++;
+	memcpy(socket, si->address_str.s, si->address_str.len);
+	socket = socket + si->address_str.len;
+	*socket = ':';
+	socket++;
+	memcpy(socket, si->port_no_str.s, si->port_no_str.len);
+	socket = socket + si->port_no_str.len;
+	*socket = 0;
 
+	LM_DBG(" --- SERVER = %s \n \n", *saddr);
 	return 1;
-
 }
 
 
@@ -881,8 +862,7 @@ end:
  */
 int find_body_pidf(struct sip_msg *msg, char** pidf_body) {
 
-	struct part* mbody_part;
-	struct multi_body *mbody;
+	struct body_part* mbody_part;
 	char *body_start, *body_end;
 	char *body_aux;
 	int size_body = 0;
@@ -890,19 +870,24 @@ int find_body_pidf(struct sip_msg *msg, char** pidf_body) {
 	UNUSED(cont);
 
 	LM_DBG(" --- FIND PIDF BODY \n \n");
-	mbody = get_all_bodies(msg);
-	if (mbody == NULL) {
+	if ( parse_sip_body(msg)<0 || msg->body==NULL) {
 		LM_ERR("Failed to get bodies\n");
 		return -1;
 	}
 
 
-	mbody_part = mbody->first;
+	mbody_part = &msg->body->first;
 	while (mbody_part != NULL) {
+
+		/* skip body parts which were deleted or newly added */
+		if (!is_body_part_received(mbody_part))
+			continue;
+
 		LM_DBG(" --- PIDF BODY %.*s", mbody_part->body.len, mbody_part->body.s);
 		LM_DBG(" --- PIDF BODY COUNT %d", ++cont);
 
-		if (strstr(mbody_part->body.s, CONTENT_TYPE_PIDF) != NULL) {
+		if ( mbody_part->mime_s.len==MIME_PIDF_LEN &&
+		memcmp(mbody_part->mime_s.s, MIME_PIDF, mbody_part->mime_s.len)==0 ) {
 			body_start = strstr(mbody_part->body.s, PRESENCE_START);
 			body_end = strstr(mbody_part->body.s, PRESENCE_END);
 			size_body = body_end - body_start + 11;
@@ -950,7 +935,7 @@ int proxy_request(struct sip_msg *msg,char *call_server_hostname) {
 		return -1;
 	}
 
-	LM_DBG(" ---USER: %.*s \n\n", msg->parsed_uri.user.len, msg->parsed_uri.user.s);
+	LM_DBG(" ---USER: %.*s \n", msg->parsed_uri.user.len, msg->parsed_uri.user.s);
 	int server_host_len = strlen(call_server_hostname);
 	size_new_uri = server_host_len + msg->parsed_uri.user.len + 6;
 
@@ -968,11 +953,11 @@ int proxy_request(struct sip_msg *msg,char *call_server_hostname) {
 	*ack_aux = '@';
 	ack_aux++;
 	memcpy(ack_aux, call_server_hostname, server_host_len);
-	LM_DBG(" ---NEW_URI: %s \n\n", ack_uri);
-	LM_DBG(" ---NEW_URI -TAM : %d \n\n", size_new_uri);
+	LM_DBG(" ---NEW_URI: %s \n", ack_uri);
+	LM_DBG(" ---NEW_URI -TAM : %d \n", size_new_uri);
 
 	if(new_uri_proxy(msg, ack_uri) == -1){
-		LM_ERR(" ---ERRO EM NEW_URI_PROXY");
+		LM_ERR(" ---ERRO EM NEW_URI_PROXY\n");
 		return -1;
 	}
 
