@@ -1027,7 +1027,7 @@ int xj_manage_jab(char *buf, int len, int *pos, xj_jalias als, xj_jcon jbc)
 		if(!strncasecmp(type, "unsubscribed", 12))
 		{
 #ifdef XJ_EXTRA_DEBUG
-			LM_DBG("user <%s> does not allow to see this"
+			LM_DBG("user <%s> does not allow one to see this"
 				" presence status\n", from);
 #endif
 			if(prc->state != XJ_PS_REFUSED)
@@ -1156,14 +1156,23 @@ int xj_send_sip_msg(str *proxy, str *to, str *from, str *msg, int *cbp)
 			|| (cbp && *cbp!=0) )
 		return -1;
 
+	if (4 + from->len > 512) {
+		LM_ERR("from too large %d\n", from->len + 4);
+		return -1;
+	}
+	
 	// from correction
 	tfrom.len = 0;
-	strncpy(buf+tfrom.len, "sip:", 4);
+	strncpy(buf+tfrom.len, "sip:", 5); /* null terminated! */
 	tfrom.len += 4;
 	strncpy(buf+tfrom.len, from->s, from->len);
 	tfrom.len += from->len;
 
 	tfrom.s = buf;
+	if (tfrom.len + 24 + CRLF_LEN + 9 + CRLF_LEN > 1024) {
+		LM_ERR("headers too large %d\n", (int)(tfrom.len + 33 + CRLF_LEN * 2));
+		return -1;
+	}
 
 	// building Contact and Content-Type
 	strcpy(buf1,"Content-Type: text/plain"CRLF"Contact: ");
