@@ -22,6 +22,7 @@
  * History:
  * ---------
  *  2009-09-04  first version (andreidragus)
+ *  2017-12-12  use opensips_json_c_helper.h (besser82)
  */
 
 
@@ -43,14 +44,9 @@
 #include "../../mi/mi.h"
 #include "../tm/tm_load.h"
 #include "../rr/api.h"
+#include "../../lib/json/opensips_json_c_helper.h"
 
 
-#include <json.h>
-#include <json_object_private.h>
-
-
-#define JSON_BUFF_SIZE 4096
-
 enum
 {
 	TAG_KEY = 1,
@@ -69,6 +65,13 @@ typedef struct _pv_json
 
 }pv_json_t;
 
+typedef struct json_object_iterator  json_iter_t;
+
+enum {
+	ITER_NONE,
+	ITER_KEYS,
+	ITER_VALUES
+};
 
 typedef struct _tag_list
 {
@@ -82,47 +85,62 @@ typedef struct _tag_list
 
 }json_tag;
 
-
 typedef struct _json_name
 {
 	str name;
 	json_tag * tags;
 	json_tag ** end;
 
+	int iter_type;
+	int iter_prev_idx;
+	json_iter_t iter;
 
 }json_name;
 
 pv_json_t * all;
-char buff[JSON_BUFF_SIZE];
+char buff[JSON_FILE_BUF_SIZE];
+int json_long_quoting;
 
 static int mod_init(void);
 static int child_init(int );
 static void mod_destroy(void);
-void json_object_array_del(struct json_object* , int );
-static int fixup_json_bind(void**, int );
+static int fixup_json_bind(void**);
 static int pv_set_json (struct sip_msg*,  pv_param_t*, int , pv_value_t* );
 static int pv_get_json (struct sip_msg*,  pv_param_t*, pv_value_t* );
-static int json_bind(struct sip_msg* , char* , char* );
+static int pv_get_json_compact(struct sip_msg*,  pv_param_t*, pv_value_t* );
+static int pv_get_json_pretty(struct sip_msg*,  pv_param_t*, pv_value_t* );
+static int pv_get_json_ext(struct sip_msg*,  pv_param_t*, pv_value_t* , int flags);
+static int json_bind(struct sip_msg* , pv_spec_t* , pv_spec_t* );
 static void print_tag_list( json_tag *, json_tag *, int);
-static json_t * get_object(pv_json_t * , pv_param_t* ,  json_tag **, int  );
-static int pv_parse_json_name (pv_spec_p , str *);
+static json_t *get_object(pv_json_t *, pv_param_t *, json_tag **, int, int);
+static int pv_parse_json_name (pv_spec_p, const str *);
+static int pv_parse_json_index(pv_spec_p sp, const str *in);
 static pv_json_t * get_pv_json (pv_param_t* );
 static int pv_add_json ( pv_param_t* , json_t * );
 static int expand_tag_list( struct sip_msg*, json_tag *);
 
 
-
-static cmd_export_t cmds[]={
-	{"json_link",    (cmd_function)json_bind,   2,
-		fixup_json_bind, 0,
+static const cmd_export_t cmds[]={
+	{"json_link",    (cmd_function)json_bind, {
+		{CMD_PARAM_VAR, fixup_json_bind, 0},
+		{CMD_PARAM_VAR, fixup_json_bind, 0}, {0,0,0}},
 		REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|
 		LOCAL_ROUTE|STARTUP_ROUTE|TIMER_ROUTE|EVENT_ROUTE},
-	{0,0,0,0,0,0}
+	{0,0,{{0,0,0}},0}
+};
+
+static const param_export_t mod_params[]={
+	{ "enable_long_quoting",         INT_PARAM, &json_long_quoting       },
+	{ 0,0,0 }
 };
 
 
-static pv_export_t mod_items[] = {
+static const pv_export_t mod_items[] = {
 	{ {"json",  sizeof("json")-1},    PVT_JSON, pv_get_json,
+		pv_set_json, pv_parse_json_name, pv_parse_json_index, 0, 0},
+	{ {"json_compact",  sizeof("json_compact")-1}, PVT_JSON, pv_get_json_compact,
+		pv_set_json, pv_parse_json_name, 0, 0, 0},
+	{ {"json_pretty",  sizeof("json_pretty")-1}, PVT_JSON, pv_get_json_pretty,
 		pv_set_json, pv_parse_json_name, 0, 0, 0},
 	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
 };
@@ -132,30 +150,31 @@ struct module_exports exports= {
 	MOD_TYPE_DEFAULT,/* class of this module */
 	MODULE_VERSION,
 	DEFAULT_DLFLAGS, /* dlopen flags */
+	0,               /* load function */
 	NULL,            /* OpenSIPS module dependencies */
 	cmds,            /* exported functions */
 	0,               /* exported async functions */
-	0,      /* param exports */
-	0,       /* exported statistics */
-	0,         /* exported MI functions */
+	mod_params,      /* param exports */
+	0,               /* exported statistics */
+	0,               /* exported MI functions */
 	mod_items,       /* exported pseudo-variables */
+	0,               /* exported transformations */
 	0,               /* extra processes */
+	0,               /* module pre-initialization function */
 	mod_init,        /* module initialization function */
 	0,               /* reply processing function */
 	mod_destroy,
-	child_init       /* per-child init function */
+	child_init,      /* per-child init function */
+	0                /* reload confirm function */
 };
 
-int json_bind(struct sip_msg* msg, char* s1, char* s2)
+int json_bind(struct sip_msg* msg, pv_spec_t* dest, pv_spec_t* src)
 {
-	pv_spec_t * src, * dest;
 	pv_json_t * var ;
 	json_t * obj;
 	json_name * id ;
 	pv_param_t *pvp;
 
-	src  = (pv_spec_t *) s2;
-	dest = (pv_spec_t *) s1;
 	pvp = &src->pvp;
 
 	id = (json_name *) pvp->pvn.u.dname;
@@ -168,7 +187,7 @@ int json_bind(struct sip_msg* msg, char* s1, char* s2)
 		return -1;
 	}
 
-	obj = get_object(var, pvp, NULL, 1);
+	obj = get_object(var, pvp, NULL, 0, 1);
 
 
 	if( obj == NULL )
@@ -185,38 +204,15 @@ int json_bind(struct sip_msg* msg, char* s1, char* s2)
 	return 1;
 };
 
-int fixup_json_bind(void** param, int param_no)
+int fixup_json_bind(void** param)
 {
-		pv_spec_t * var;
-		char * ret;
-		str s;
-		s.s = *param;
-		s.len = strlen(s.s);
+	if(pv_type(((pv_spec_t*)*param)->type) != PVT_JSON)
+	{
+		LM_ERR("Parameter must be a json variable\n");
+		return -1;
+	}
 
-		var = (pv_spec_t *)pkg_malloc(sizeof(pv_spec_t));
-		if( var == NULL )
-		{
-			LM_ERR("Out of memory\n");
-			return -1;
-		}
-
-		ret = pv_parse_spec(&s,var);
-
-		if( ret == NULL )
-		{
-			LM_ERR("Parse error\n");
-			return -1;
-		}
-
-		if( var->type != PVT_JSON )
-		{
-			LM_ERR("Parameter no: %d must be a json variable\n",param_no);
-			return -1;
-		}
-
-		*param = var;
-
-		return 0;
+	return 0;
 }
 
 
@@ -268,21 +264,17 @@ pv_json_t * get_pv_json (pv_param_t* pvp)
 
 
 
-json_t * get_object(pv_json_t * var, pv_param_t* pvp ,  json_tag ** tag,
-			int report_err )
+json_t *get_object(pv_json_t *var, pv_param_t *pvp, json_tag **tag,
+				int get_prev_obj, int report_err)
 {
 	json_name * id = (json_name *) pvp->pvn.u.dname;
-
 	json_t * cur_obj, * last_obj = 0;
 	json_tag * cur_tag, * last_tag = 0;
 	int poz;
 
-
-
 	cur_tag = id->tags;
 	cur_obj = var->data;
 
-
 	while( cur_tag  )
 	{
 		last_tag = cur_tag;
@@ -297,19 +289,9 @@ json_t * get_object(pv_json_t * var, pv_param_t* pvp ,  json_tag ** tag,
 				!json_object_is_type( cur_obj, json_type_object ) )
 				goto error;
 
-#if JSON_LIB_VERSION < 10
-			cur_obj = json_object_object_get( cur_obj, buff );
-
-			if( cur_obj == NULL && tag == NULL)
-				goto error;
-#else
 			if (!json_object_object_get_ex( cur_obj,buff, &cur_obj ) &&
-				tag == NULL)
+				!get_prev_obj)
 				goto error;
-#endif
-
-
-
 		}
 
 		if( cur_tag->type & TAG_IDX )
@@ -338,7 +320,7 @@ json_t * get_object(pv_json_t * var, pv_param_t* pvp ,  json_tag ** tag,
 
 			cur_obj = json_object_array_get_idx( cur_obj, poz );
 
-			if( cur_obj == NULL && tag == NULL)
+			if( cur_obj == NULL && !get_prev_obj)
 				goto error;
 
 		}
@@ -346,15 +328,13 @@ json_t * get_object(pv_json_t * var, pv_param_t* pvp ,  json_tag ** tag,
 		cur_tag = cur_tag->next;
 	}
 
-	if( tag == NULL )
-	{
-		return cur_obj;
-	}
-	else
-	{
+	if (tag)
 		*tag = last_tag;
+
+	if (get_prev_obj)
 		return last_obj;
-	}
+	else
+		return cur_obj;
 
 error:
 
@@ -373,15 +353,76 @@ error:
 
 }
 
+int pv_get_json(struct sip_msg* msg,  pv_param_t* pvp, pv_value_t* val)
+{
+	return pv_get_json_ext(msg, pvp, val, JSON_C_TO_STRING_SPACED);
+}
 
-int pv_get_json (struct sip_msg* msg,  pv_param_t* pvp, pv_value_t* val)
+int pv_get_json_compact(struct sip_msg* msg,  pv_param_t* pvp, pv_value_t* val)
+{
+	return pv_get_json_ext(msg, pvp, val, JSON_C_TO_STRING_PLAIN);
+}
+
+int pv_get_json_pretty(struct sip_msg* msg,  pv_param_t* pvp, pv_value_t* val)
+{
+	return pv_get_json_ext(msg, pvp, val, JSON_C_TO_STRING_PRETTY);
+}
+
+int pv_json_iterate(json_t **obj, pv_param_t *pvp, json_name *id, pv_value_t *val)
+{
+	json_iter_t iter_end;
+
+	if (json_object_is_type(*obj, json_type_object)) {
+
+		if (pvp->pvi.u.ival != id->iter_prev_idx + 1) {
+			id->iter_prev_idx = 0;
+			id->iter = json_object_iter_begin(*obj);
+		} else
+			id->iter_prev_idx++;
+
+		iter_end = json_object_iter_end(*obj);
+		if (json_object_iter_equal(&id->iter, &iter_end))
+			return pv_get_null(NULL, pvp, val);
+
+		if (id->iter_type == ITER_KEYS) {
+			val->flags = PV_VAL_STR;
+			val->rs.s = (char *)json_object_iter_peek_name(&id->iter);
+			val->rs.len = strlen(val->rs.s);
+		} else
+			*obj = json_object_iter_peek_value(&id->iter);
+
+		json_object_iter_next(&id->iter);	
+
+	} else if (json_object_is_type(*obj, json_type_array)) {
+
+		if (id->iter_type != ITER_NONE) {
+			LM_DBG("Invalid object-like iteration for arrays\n");
+			return -1;
+		}
+
+		if (pvp->pvi.u.ival == json_object_array_length(*obj)) {
+			id->iter_prev_idx = 0;
+			return pv_get_null(NULL, pvp, val);
+		}
+
+		*obj = json_object_array_get_idx(*obj, pvp->pvi.u.ival);
+
+	} else {
+		LM_DBG("Can only iterate over arrays or objects\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+int pv_get_json_ext(struct sip_msg* msg,  pv_param_t* pvp, pv_value_t* val, int flags)
 {
 
 	pv_json_t * var ;
 	json_t * obj;
 	json_name * id = (json_name *) pvp->pvn.u.dname;
 	UNUSED(id);
-
+	int64_t int_value;
 
 	if( expand_tag_list( msg, ((json_name *)pvp->pvn.u.dname)->tags ) < 0)
 	{
@@ -400,31 +441,54 @@ int pv_get_json (struct sip_msg* msg,  pv_param_t* pvp, pv_value_t* val)
 		return pv_get_null( msg, pvp, val);
 	}
 
-	obj = get_object(var, pvp, NULL, 0);
+	obj = get_object(var, pvp, NULL, 0, 0);
+
 	memset(val, 0, sizeof(pv_value_t));
 
 	if( obj == NULL )
 		return pv_get_null( msg, pvp, val);
 
+	if (pvp->pvi.type == PV_IDX_INT) {
+		if (pv_json_iterate(&obj, pvp, id, val) < 0) {
+			LM_DBG("Failed to iterate\n");
+			return pv_get_null(msg, pvp, val);
+		}
+
+		if (val->flags == PV_VAL_STR || val->flags == PV_VAL_NULL)
+			/* val is set */
+			return 0;
+		/* else we got an object */
+	} else if (pvp->pvi.type == PV_IDX_ALL) {
+		LM_ERR("\"[*]\" index only supported in for each statement\n");
+		return pv_get_null(msg, pvp, val);
+	}
+
 	if( json_object_is_type(obj, json_type_int) )
 	{
-		val->rs.s = sint2str(json_object_get_int(obj), &val->rs.len);
-		val->ri = json_object_get_int(obj);;
-		val->flags |= PV_VAL_INT|PV_TYPE_INT|PV_VAL_STR;
+		int_value = json_object_get_int64(obj);
+		val->rs.s = sint2str(int_value, &val->rs.len);
 
+		if (!json_long_quoting || (int_value>=INT_MIN && int_value<=INT_MAX)) {
+			/* safe to store it as an INT in the pvar */
+			val->ri = int_value;
+			val->flags |= PV_VAL_INT|PV_TYPE_INT|PV_VAL_STR;
+		} else {
+			/* we would overflow/underflow, store as string only */
+			val->flags |= PV_VAL_STR;
+		}
 	}
 	else if( json_object_is_type(obj, json_type_string))
 	{
 		val->flags = PV_VAL_STR;
 		val->rs.s = (char*)json_object_get_string( obj );
-#if JSON_LIB_VERSION >= 10
+#if JSON_C_VERSION_NUM >= JSON_C_VERSION_010
 		val->rs.len = json_object_get_string_len( obj );
 #else
 		val->rs.len = strlen(val->rs.s);
 #endif
 	} else {
 		val->flags = PV_VAL_STR;
-		val->rs.s = (char*)json_object_to_json_string( obj );
+		val->rs.s = (char*)json_object_to_json_string_ext( obj, flags);
 		val->rs.len = strlen(val->rs.s);
 	}
 
@@ -484,7 +548,7 @@ int pv_add_json ( pv_param_t* pvp, json_t * obj )
 	}
 
 
-	dest = get_object(var, pvp, &tag, 1);
+	dest = get_object(var, pvp, &tag, 1, 1);
 
 	if( dest == NULL )
 	{
@@ -589,7 +653,7 @@ int pv_set_json (struct sip_msg* msg,  pv_param_t* pvp, int flag ,
 		if (obj == NULL)
 		{
 			LM_ERR("Error parsing json: %s\n",
-#if JSON_LIB_VERSION >= 10
+#if JSON_C_VERSION_NUM >= JSON_C_VERSION_010
 				json_tokener_error_desc(parse_status)
 #else
 				json_tokener_errors[(unsigned long)obj]
@@ -604,7 +668,7 @@ int pv_set_json (struct sip_msg* msg,  pv_param_t* pvp, int flag ,
 	}
 	else
 	{
-		if( val->flags & PV_VAL_INT )
+		if( pvv_is_int(val))
 		{
 			obj = json_object_new_int(val->ri);
 		}
@@ -628,11 +692,14 @@ enum
 	ST_TEST = 1,
 	ST_KEY = 2,
 	ST_IDX = 3,
-	ST_ERR = 4
+	ST_ITER = 4,
+	ST_ERR = 5
 };
 
-int next[4][256];
-int ignore[4][256];
+#define NO_VALID_STATES 5
+
+int next[NO_VALID_STATES][256];
+int ignore[NO_VALID_STATES][256];
 int inited;
 
 int expand_tag_list( struct sip_msg* msg,json_tag * start)
@@ -741,6 +808,8 @@ int get_value(int state, json_name * id, char *start, char * cur)
 	char * i;
 	int empty;
 	str in;
+	static str keys_s = str_init("keys");
+	static str values_s = str_init("values");
 
 	in.s = start;
 	in.len = cur-start;
@@ -766,14 +835,15 @@ int get_value(int state, json_name * id, char *start, char * cur)
 
 			memset(node,0,sizeof(json_tag));
 			node->type = TAG_KEY;
+
 			*id->end = node;
 			id->end = &node->next;
 
 			if( in.len > 0 && *start == '$' )
 			{
-				if( pv_parse_spec(&in, &node->var) < 0 )
+				if(!pv_parse_spec(&in, &node->var))
 				{
-					LM_ERR("Unable to parse variable ");
+					LM_ERR("Unable to parse variable\n");
 					return -1;
 				}
 
@@ -818,9 +888,9 @@ int get_value(int state, json_name * id, char *start, char * cur)
 
 			if( *i == '$' )
 			{
-				if( pv_parse_spec(&in, &node->var) < 0 )
+				if(!pv_parse_spec(&in, &node->var))
 				{
-					LM_ERR("Unable to parse variable ");
+					LM_ERR("Unable to parse variable\n");
 					return -1;
 				}
 
@@ -836,8 +906,17 @@ int get_value(int state, json_name * id, char *start, char * cur)
 			}
 
 			break;
+		case ST_ITER:
+			if (str_match(&keys_s, &in))
+				id->iter_type = ITER_KEYS;
+			else if (str_match(&values_s, &in))
+				id->iter_type = ITER_VALUES;
+			else {
+				LM_ERR("Bad iterator type\n");
+				return -1;
+			}
 
-
+			break;
 	}
 	return 0;
 }
@@ -847,26 +926,31 @@ void init_matrix(void)
 {
 	int i,j;
 
-
 	/* point each state to itself */
-	for( i=0; i<4; i++)
+	for( i=0; i<NO_VALID_STATES; i++)
 		for( j=0; j<256; j++)
 			next[i][j] = i;
 
 	next[ST_NAME][(unsigned int)'/'] = ST_TEST;
 	next[ST_NAME][(unsigned int)'['] = ST_TEST;
+	next[ST_NAME][(unsigned int)'.'] = ST_TEST;
 
 	for( j=0; j<256; j++)
 		next[ST_TEST][j] = ST_ERR;
 
 	next[ST_TEST][(unsigned int)'['] = ST_IDX;
 	next[ST_TEST][(unsigned int)'/'] = ST_KEY;
+	next[ST_TEST][(unsigned int)'.'] = ST_ITER;
 
 	next[ST_IDX][(unsigned int)']'] = ST_TEST;
 
-
 	next[ST_KEY][(unsigned int)'['] = ST_TEST;
 	next[ST_KEY][(unsigned int)'/'] = ST_TEST;
+	next[ST_KEY][(unsigned int)'.'] = ST_TEST;
+
+	next[ST_ITER][(unsigned int)'['] = ST_ERR;
+	next[ST_ITER][(unsigned int)'/'] = ST_ERR;
+	next[ST_ITER][(unsigned int)'.'] = ST_ERR;
 
 	/* set chars that will not be consumed */
 	for( j=0; j<256; j++)
@@ -880,13 +964,14 @@ void init_matrix(void)
 	ignore[ST_TEST][(unsigned int)'['] = 0;
 	ignore[ST_KEY][(unsigned int)'['] = 1;
 
-
-
+	ignore[ST_NAME][(unsigned int)'.'] = 1;
+	ignore[ST_TEST][(unsigned int)'.'] = 0;
+	ignore[ST_KEY][(unsigned int)'.'] = 1;
 }
 
 
 
-int pv_parse_json_name (pv_spec_p sp, str *in)
+int pv_parse_json_name (pv_spec_p sp, const str *in)
 {
 	json_name * id;
 	char * cur,* start;
@@ -902,8 +987,8 @@ int pv_parse_json_name (pv_spec_p sp, str *in)
 		LM_ERR("Out of memory\n");
 		return -1;
 	}
+	memset(id, 0, sizeof *id);
 
-	id->tags = NULL;
 	id->end = &id->tags;
 
 
@@ -950,24 +1035,27 @@ int pv_parse_json_name (pv_spec_p sp, str *in)
 	if( get_value(state, id, start, cur) )
 		return -1;
 
-
 	sp->pvp.pvn.u.dname = id ;
-	sp->type = PVT_JSON;
-	sp->getf = pv_get_json;
-	sp->setf = pv_set_json;
-
 
 	return 0;
 }
 
+static int pv_parse_json_index(pv_spec_p sp, const str *in)
+{
+	if (in == NULL || in->s == NULL || sp == NULL)
+		return -1;
 
-
-
-
+	if (*in->s == '*' && in->len == 1) {
+		sp->pvp.pvi.type = PV_IDX_ALL;
+		return 0;
+	} else {
+		LM_ERR("The only index supported is \"[*]\" in for each statements\n");
+		return -1;
+	}
+}
 
 int mod_init(void)
 {
-
 	return 0;
 }
 
@@ -980,4 +1068,3 @@ void mod_destroy(void)
 {
 
 }
-
