@@ -178,7 +178,6 @@ static inline void ws_print_masked(char *buf, int len)
 			p += sprintf(p, "\n");
 		}
 	}
-	LM_INFO("Print buffer\n%s", print_buf);
 }
 
 static inline void ws_mask(char *buf, int len, unsigned int mask)
@@ -330,7 +329,8 @@ static enum ws_close_code inline ws_parse(struct ws_req *req)
 			/* extended case */
 			if (req->tcp.pos - req->tcp.buf < WS_MIN_HDR_LEN + WS_ELENC_SIZE +
 					WS_IF_MASK_SIZE(req))
-				return 0;
+				/* wait for more data to come */
+				goto update_parsed;
 
 			clen = WS_ELENC(req);
 			if ((clen+WS_MIN_HDR_LEN+WS_ELENC_SIZE+WS_IF_MASK_SIZE(req))>
@@ -346,7 +346,8 @@ static enum ws_close_code inline ws_parse(struct ws_req *req)
 			/* extended case */
 			if (req->tcp.pos - req->tcp.buf < WS_MIN_HDR_LEN + WS_ELEN_SIZE +
 					WS_IF_MASK_SIZE(req))
-				return 0;
+				/* wait for more data to come */
+				goto update_parsed;
 
 			req->tcp.content_len = WS_ELEN(req);
 			if ((req->tcp.content_len+WS_MIN_HDR_LEN+WS_ELEN_SIZE+WS_IF_MASK_SIZE(req))>
@@ -458,8 +459,6 @@ again:
 		tcp_conn_set_lifetime(con, _ws_common_write_tout);
 		con->timeout=con->lifetime;
 
-		/* if we are here everything is nice and ok*/
-		update_stat( pt[process_no].load, +1 );
 		/* rcv.bind_address should always be !=0 */
 		bind_address=con->rcv.bind_address;
 
@@ -531,7 +530,7 @@ again:
 					"keeping connection \n");
 			}
 
-			if (receive_msg(msg_buf, msg_len, &local_rcv, NULL) <0)
+			if (receive_msg(msg_buf, msg_len, &local_rcv, NULL, 0) <0)
 					LM_ERR("receive_msg failed \n");
 
 			*req->tcp.parsed = bk;
@@ -543,8 +542,6 @@ again:
 				goto error;
 			}
 
-		update_stat( pt[process_no].load, -1 );
-
 		if (size) memmove(req->tcp.buf, req->tcp.parsed, size);
 #ifdef EXTRA_DEBUG
 		LM_DBG("preparing for new request, kept %ld bytes\n", size);
@@ -559,7 +556,7 @@ again:
 		if (req != &_ws_common_current_req) {
 			/* make sure we cleanup the request in the connection */
 			con->con_req = NULL;
-			pkg_free(req);
+			shm_free(req);
 		}
 
 	} else {
@@ -576,7 +573,7 @@ again:
 			/* let's duplicate this - most likely another conn will come in */
 
 			LM_DBG("We didn't manage to read a full request\n");
-			newreq = pkg_malloc(sizeof(struct ws_req));
+			newreq = shm_malloc(sizeof(struct ws_req));
 			if (newreq == NULL) {
 				LM_ERR("No more mem for dynamic con request buffer\n");
 				goto error;
@@ -638,4 +635,91 @@ static void ws_close(struct tcp_connection *c)
 	ws_send_close(c);
 }
 
+static struct tcp_connection* ws_sync_connect(struct socket_info* send_sock,
+		union sockaddr_union* server, struct tcp_conn_profile *prof)
+{
+	int s;
+	union sockaddr_union my_name;
+	socklen_t my_name_len;
+	struct tcp_connection* con;
+
+	s=socket(AF2PF(server->s.sa_family), SOCK_STREAM, 0);
+	if (s==-1){
+		LM_ERR("socket: (%d) %s\n", errno, strerror(errno));
+		goto error;
+	}
+
+	if (tcp_init_sock_opt(s, prof, send_sock->flags)<0){
+		LM_ERR("tcp_init_sock_opt failed\n");
+		goto error;
+	}
+
+	my_name_len = sockaddru_len(send_sock->su);
+	memcpy( &my_name, &send_sock->su, my_name_len);
+	su_setport( &my_name, 0);
+	if (bind(s, &my_name.s, my_name_len )!=0) {
+		LM_ERR("bind failed (%d) %s\n", errno,strerror(errno));
+		goto error;
+	}
+
+	if (tcp_connect_blocking_timeout(s, &server->s, sockaddru_len(*server),
+	                      prof->connect_timeout)<0){
+		LM_ERR("tcp_blocking_connect failed\n");
+		goto error;
+	}
+	con=tcp_conn_create(s, server, send_sock, prof, S_CONN_OK, 0);
+	if (con==NULL){
+		LM_ERR("tcp_conn_create failed, closing the socket\n");
+		goto error;
+	}
+	/* it is safe to move this here and clear it after we complete the
+	 * handshake, just before sending the fd to main */
+	con->fd = s;
+	return con;
+error:
+	/* close the opened socket */
+	if (s!=-1) close(s);
+	return 0;
+}
+
+
+static struct tcp_connection* ws_connect(struct socket_info* send_sock,
+		union sockaddr_union* to, struct tcp_conn_profile *prof, int *fd)
+{
+	struct tcp_connection *c;
+
+	if ((c=ws_sync_connect(send_sock, to, prof))==0) {
+		LM_ERR("connect failed\n");
+		return NULL;
+	}
+	/* the state of the connection should be NONE, otherwise something is
+	 * wrong */
+	if (WS_TYPE(c) != WS_NONE) {
+		LM_BUG("invalid type for connection %d\n", WS_TYPE(c));
+		goto error;
+	}
+	WS_TYPE(c) = WS_CLIENT;
+
+	if (ws_client_handshake(c) < 0) {
+		LM_ERR("cannot complete WebSocket handshake\n");
+		goto error;
+	}
+
+	*fd = c->fd;
+	/* clear the fd, just in case */
+	c->fd = -1;
+	/* handshake done - send the socket to main */
+	if (tcp_conn_send(c) < 0) {
+		LM_ERR("cannot send socket to main\n");
+		goto error;
+	}
+
+	return c;
+error:
+	tcp_conn_destroy(c);
+	return NULL;
+}
+
+
+
 #endif /* _WS_COMMON_H_ */
