@@ -405,7 +405,7 @@ static int init_transfer(CURL *handle, char *url, unsigned long timeout_s)
 
 	w_curl_easy_setopt(handle, CURLOPT_URL, url);
 	if (curl_http_version != CURL_HTTP_VERSION_NONE)
-		w_curl_easy_setopt(handle, CURLOPT_HTTP_VERSION, curl_http_version);
+		w_curl_easy_setopt(handle, CURLOPT_HTTP_VERSION, (long)curl_http_version);
 
 	if (tls_dom) {
 		w_curl_easy_setopt(handle, CURLOPT_SSLCERT, tls_dom->cert.s);
@@ -419,9 +419,8 @@ static int init_transfer(CURL *handle, char *url, unsigned long timeout_s)
 	w_curl_easy_setopt(handle, CURLOPT_TIMEOUT,
 			timeout_s && timeout_s < curl_timeout ? timeout_s : curl_timeout);
 
-	w_curl_easy_setopt(handle, CURLOPT_VERBOSE, 1);
-	w_curl_easy_setopt(handle, CURLOPT_STDERR, stdout);
-	w_curl_easy_setopt(handle, CURLOPT_FAILONERROR, 0);
+	w_curl_easy_setopt(handle, CURLOPT_VERBOSE, 0L);
+	w_curl_easy_setopt(handle, CURLOPT_FAILONERROR, 0L);
 
 	if (ssl_capath)
 		w_curl_easy_setopt(handle, CURLOPT_CAPATH, ssl_capath);
@@ -477,7 +476,7 @@ static inline int set_upload_opts(CURL *handle, str *ctype, str *body)
 	 *	   strings (e.g. $du), thus curl's strlen() may overflow or crash
 	 */
 #if (LIBCURL_VERSION_NUM >= 0x071101)
-	w_curl_easy_setopt(handle, CURLOPT_POSTFIELDSIZE, body->len);
+	w_curl_easy_setopt(handle, CURLOPT_POSTFIELDSIZE, (long)body->len);
 	w_curl_easy_setopt(handle, CURLOPT_COPYPOSTFIELDS, body->s);
 #else
 	w_curl_easy_setopt(handle, CURLOPT_POSTFIELDS, body->s);
@@ -490,7 +489,7 @@ cleanup:
 
 #define set_post_opts(handle, ctype, body) \
 	do { \
-		w_curl_easy_setopt(handle, CURLOPT_POST, 1); \
+		w_curl_easy_setopt(handle, CURLOPT_POST, 1L); \
 		if (set_upload_opts(handle, ctype, body) != 0) { \
 			LM_ERR("failed to init POST to %s\n", url); \
 			goto cleanup; \
@@ -820,8 +819,8 @@ int start_async_http_req(struct sip_msg *msg, enum rest_client_method method,
 	CURLcode rc;
 	CURLMcode mrc;
 	fd_set rset, wset, eset;
-	int max_fd, fd, http_rc, ret = RCL_INTERNAL_ERR;
-	long busy_wait, timeout, connect_timeout;
+	int max_fd, fd, ret = RCL_INTERNAL_ERR;
+	long http_rc, busy_wait, timeout, connect_timeout;
 	long retry_time;
 	OSS_CURLM *multi_list;
 	CURLM *multi_handle;
@@ -974,7 +973,7 @@ int start_async_http_req(struct sip_msg *msg, enum rest_client_method method,
 			for (fd = 0; fd <= max_fd; fd++) {
 				if (FD_ISSET(fd, &rset)) {
 					LM_DBG("ongoing transfer on fd %d\n", fd);
-					if (connect > 0 && req_sz > 0 && is_new_transfer(fd)) {
+					if ((connect > 0 || req_sz > 0) && is_new_transfer(fd)) {
 						LM_DBG(">>> add fd %d to ongoing transfers\n", fd);
 						add_transfer(fd);
 						goto success;
