@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2001-2003 FhG Fokus
+ * Copyright (C) 2020 OpenSIPS Solutions
  *
  * This file is part of opensips, a free SIP server.
  *
@@ -15,21 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
- *
- * History:
- * ---------
- * 2003-03-11 changed to the new locking scheme: locking.h (andrei)
- * 2003-03-12 added replication mark and zombie state (nils)
- * 2004-06-07 updated to the new DB api (andrei)
- * 2004-08-23  hash function changed to process characters as unsigned
- *             -> no negative results occur (jku)
- *
- */
-
-/*! \file
- *  \brief USRLOC -
- *  \ingroup usrloc
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <string.h>
@@ -43,12 +30,18 @@
 #include "../../socket_info.h"
 #include "../../ut.h"
 #include "../../hash_func.h"
+#include "../../cachedb/cachedb.h"
+
 #include "ul_mod.h"            /* usrloc module parameters */
+#include "ul_evi.h"
 #include "utime.h"
-#include "ureplication.h"
+#include "ul_cluster.h"
+#include "ul_callback.h"
+#include "usrloc.h"
 
 
 extern int max_contact_delete;
+extern int cid_regen;
 extern db_key_t *cid_keys;
 extern db_val_t *cid_vals;
 int cid_len=0;
@@ -126,170 +119,6 @@ error0:
 }
 
 
-static event_id_t ei_ins_id = EVI_ERROR;
-static event_id_t ei_del_id = EVI_ERROR;
-event_id_t ei_c_ins_id = EVI_ERROR;
-event_id_t ei_c_del_id = EVI_ERROR;
-event_id_t ei_c_update_id = EVI_ERROR;
-static str ei_ins_name = str_init("E_UL_AOR_INSERT");
-static str ei_del_name = str_init("E_UL_AOR_DELETE");
-static str ei_contact_ins_name = str_init("E_UL_CONTACT_INSERT");
-static str ei_contact_del_name = str_init("E_UL_CONTACT_DELETE");
-static str ei_contact_update_name = str_init("E_UL_CONTACT_UPDATE");
-static str ei_aor_name = str_init("aor");
-static str ei_c_addr_name = str_init("address");
-static str ei_c_recv_name = str_init("received");
-static str ei_callid_name = str_init("callid");
-static str ei_cseq_name = str_init("cseq");
-static evi_params_p ul_event_params;
-static evi_param_p ul_aor_param;
-static evi_params_p ul_contact_event_params;
-static evi_param_p ul_c_addr_param, ul_c_callid_param;
-static evi_param_p ul_c_aor_param;
-static evi_param_p ul_c_recv_param;
-static evi_param_p ul_c_cseq_param;
-
-/*! \brief
- * Initialize event structures
- */
-int ul_event_init(void)
-{
-	ei_ins_id = evi_publish_event(ei_ins_name);
-	if (ei_ins_id == EVI_ERROR) {
-		LM_ERR("cannot register aor insert event\n");
-		return -1;
-	}
-
-	ei_del_id = evi_publish_event(ei_del_name);
-	if (ei_del_id == EVI_ERROR) {
-		LM_ERR("cannot register aor delete event\n");
-		return -1;
-	}
-
-	ei_c_ins_id = evi_publish_event(ei_contact_ins_name);
-	if (ei_c_ins_id == EVI_ERROR) {
-		LM_ERR("cannot register contact insert event\n");
-		return -1;
-	}
-
-	ei_c_del_id = evi_publish_event(ei_contact_del_name);
-	if (ei_c_del_id == EVI_ERROR) {
-		LM_ERR("cannot register contact delete event\n");
-		return -1;
-	}
-
-	ei_c_update_id = evi_publish_event(ei_contact_update_name);
-	if (ei_c_update_id == EVI_ERROR) {
-		LM_ERR("cannot register contact delete event\n");
-		return -1;
-	}
-
-	ul_event_params = pkg_malloc(sizeof(evi_params_t));
-	if (!ul_event_params) {
-		LM_ERR("no more pkg memory\n");
-		return -1;
-	}
-	memset(ul_event_params, 0, sizeof(evi_params_t));
-	ul_aor_param = evi_param_create(ul_event_params, &ei_aor_name);
-	if (!ul_aor_param) {
-		LM_ERR("cannot create AOR parameter\n");
-		return -1;
-	}
-
-	ul_contact_event_params = pkg_malloc(sizeof(evi_params_t));
-	if (!ul_contact_event_params) {
-		LM_ERR("no more pkg memory\n");
-		return -1;
-	}
-	memset(ul_contact_event_params, 0, sizeof(evi_params_t));
-
-	ul_c_addr_param = evi_param_create(ul_contact_event_params, &ei_c_addr_name);
-	if (!ul_c_addr_param) {
-		LM_ERR("cannot create contact address parameter\n");
-		return -1;
-	}
-
-	ul_c_aor_param = evi_param_create(ul_contact_event_params, &ei_aor_name);
-	if (!ul_c_aor_param) {
-		LM_ERR("cannot create contact aor parameter\n");
-		return -1;
-	}
-
-	ul_c_callid_param = evi_param_create(ul_contact_event_params, &ei_callid_name);
-	if (!ul_c_callid_param) {
-		LM_ERR("cannot create callid parameter\n");
-		return -1;
-	}
-
-	ul_c_recv_param = evi_param_create(ul_contact_event_params, &ei_c_recv_name);
-	if (!ul_c_recv_param) {
-		LM_ERR("cannot create received parameter\n");
-		return -1;
-	}
-
-	ul_c_cseq_param = evi_param_create(ul_contact_event_params, &ei_cseq_name);
-	if (!ul_c_cseq_param) {
-		LM_ERR("cannot create cseq parameter\n");
-		return -1;
-	}
-
-	return 0;
-}
-
-/*! \brief
- * Raise an event when an AOR is inserted/deleted
- */
-static void ul_raise_event(event_id_t _e, struct urecord* _r)
-{
-	if (_e == EVI_ERROR) {
-		LM_ERR("event not yet registered %d\n", _e);
-		return;
-	}
-	if (evi_param_set_str(ul_aor_param, &_r->aor) < 0) {
-		LM_ERR("cannot set AOR parameter\n");
-		return;
-	}
-	if (evi_raise_event(_e, ul_event_params) < 0)
-		LM_ERR("cannot raise event\n");
-}
-
-void ul_raise_contact_event(event_id_t _e, str *addr, str *callid, str *recv,
-		str *aor, int cseq)
-{
-	if (_e == EVI_ERROR) {
-		LM_ERR("event not yet registered %d\n", _e);
-		return;
-	}
-	if (evi_param_set_str(ul_c_addr_param, addr) < 0) {
-		LM_ERR("cannot set contact address parameter\n");
-		return;
-	}
-
-	if (evi_param_set_str(ul_c_aor_param, aor) < 0) {
-		LM_ERR("cannot set contact aor parameter\n");
-		return;
-	}
-
-	if (evi_param_set_str(ul_c_callid_param, callid) < 0) {
-		LM_ERR("cannot set callid parameter\n");
-		return;
-	}
-
-	if (evi_param_set_str(ul_c_recv_param, recv) < 0) {
-		LM_ERR("cannot set received parameter\n");
-		return;
-	}
-
-	if (evi_param_set_int(ul_c_cseq_param, &cseq) < 0) {
-		LM_ERR("cannot set cseq parameter\n");
-		return;
-	}
-
-	if (evi_raise_event(_e, ul_contact_event_params) < 0)
-		LM_ERR("cannot raise event\n");
-}
-
-
 /*! \brief
  * Free all memory allocated for
  * the domain
@@ -299,11 +128,8 @@ void free_udomain(udomain_t* _d)
 	int i;
 
 	if (_d->table) {
-		for(i = 0; i < _d->size; i++) {
-			lock_ulslot(_d, i);
+		for(i = 0; i < _d->size; i++)
 			deinit_slot(_d->table + i);
-			unlock_ulslot(_d, i);
-		}
 		shm_free(_d->table);
 	}
 	shm_free(_d);
@@ -313,62 +139,139 @@ void free_udomain(udomain_t* _d)
  * Returns a static dummy urecord for temporary usage
  */
 static inline void
-get_static_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
+get_static_urecord(const udomain_t* _d, const str* _aor, struct urecord** _r)
 {
-	static struct urecord r;
+	static struct urecord r = {
+		.is_static = 1,
+	};
+
+	free_urecord(&r);
+	memset(&r, 0, sizeof r);
 
-	free_urecord( &r );
-	memset( &r, 0, sizeof(struct urecord) );
 	r.aor = *_aor;
 	r.domain = _d->name;
-	r.aorhash = core_hash(_aor, 0, 0)&(_d->size-1);
+	r.aorhash = core_hash(_aor, NULL, DB_AOR_HASH_MASK);
+	r.is_static = 1;
 
 	*_r = &r;
 }
 
 /*! \brief
- * Just for debugging
+ * expects (UL_COLS - 4) fields:
+ *   contact, expires, q, callid, cseq, flags, cflags, ua,
+ *   received, path, socket, methods, last_modified, instance, attr)
  */
-void print_udomain(FILE* _f, udomain_t* _d)
-{
-		int i;
-	int max=0, slot=0, n=0,count;
-	map_iterator_t it;
-	fprintf(_f, "---Domain---\n");
-	fprintf(_f, "name : '%.*s'\n", _d->name->len, ZSW(_d->name->s));
-	fprintf(_f, "size : %d\n", _d->size);
-	fprintf(_f, "table: %p\n", _d->table);
-	/*fprintf(_f, "lock : %d\n", _d->lock); -- can be a structure --andrei*/
-	fprintf(_f, "\n");
-	for(i=0; i<_d->size; i++)
-	{
-		count = map_size( _d->table[i].records);
-		n += count;
-		if(max<count){
-			max= count;
-			slot = i;
-		}
-
-		for ( map_first( _d->table[i].records, &it);
-			iterator_is_valid(&it);
-			iterator_next(&it) )
-			print_urecord(_f, (struct urecord *)*iterator_val(&it));
-
-	}
-
-	fprintf(_f, "\nMax slot: %d (%d/%d)\n", max, slot, n);
-	fprintf(_f, "\n---/Domain---\n");
-}
-
-
-/*! \brief
- * expects 15 rows (contact_id, contact, expires, q, callid, cseq, flags, cflags,
- *   ua, received, path, socket, methods, last_modified, instance)
- */
-static inline ucontact_info_t* dbrow2info( db_val_t *vals, str *contact)
+static inline ucontact_info_t *
+cdb_ctdict2info(const cdb_dict_t *ct_fields, str *contact)
 {
 	static ucontact_info_t ci;
-	static str callid, ua, received, host, path, instance, attr, flags;
+	static str callid, ua, received, host, path, instance;
+	static str attr;
+	struct list_head *_;
+
+	cdb_pair_t *pair;
+	int port, proto;
+
+	memset(&ci, 0, sizeof(ucontact_info_t));
+
+	/* TODO: find a less convoluted way of implementing this */
+	list_for_each (_, ct_fields) {
+		pair = list_entry(_, cdb_pair_t, list);
+
+		switch (pair->key.name.s[0]) {
+		case 'a':
+			attr = pair->val.val.st;
+			ci.attr = &attr;
+			break;
+		case 'c':
+			switch (pair->key.name.s[1]) {
+			case 'a':
+				callid = pair->val.val.st;
+				ci.callid = &callid;
+				break;
+			case 'f':
+				ci.cflags = flag_list_to_bitmask(str2const(&pair->val.val.st),
+				                                 FLAG_TYPE_BRANCH, FLAG_DELIM, 0);
+				break;
+			case 'o':
+				*contact = pair->val.val.st;
+				break;
+			case 's':
+				ci.cseq = pair->val.val.i32;
+				break;
+			}
+			break;
+		case 'e':
+			ci.expires = pair->val.val.i32;
+			break;
+		case 'f':
+			ci.flags = pair->val.val.i32;
+			break;
+		case 'l':
+			ci.last_modified = pair->val.val.i64;
+			break;
+		case 'm':
+			if (pair->val.type == CDB_NULL)
+				ci.methods = ALL_METHODS;
+			else
+				ci.methods = pair->val.val.i32;
+			break;
+		case 'p':
+			path = pair->val.val.st;
+			ci.path = &path;
+			break;
+		case 'q':
+			ci.q = pair->val.val.i32;
+			break;
+		case 'r':
+			received = pair->val.val.st;
+			ci.received = received;
+			break;
+		case 's':
+			switch (pair->key.name.s[1]) {
+			case 'i':
+				instance = pair->val.val.st;
+				ci.instance = instance;
+				break;
+			case 'o':
+				if (ZSTR(pair->val.val.st)) {
+					ci.sock = NULL;
+				} else {
+					if (parse_phostport(pair->val.val.st.s, pair->val.val.st.len,
+					                    &host.s, &host.len, &port, &proto) != 0) {
+						LM_ERR("bad socket <%.*s>\n", pair->val.val.st.len,
+						       pair->val.val.st.s);
+						return NULL;
+					}
+
+					ci.sock = grep_sock_info(&host, (unsigned short)port, proto);
+					if (!ci.sock)
+						LM_DBG("non-local socket <%.*s>...ignoring\n",
+						       pair->val.val.st.len, pair->val.val.st.s);
+				}
+				break;
+			}
+			break;
+		case 'u':
+			ua = pair->val.val.st;
+			ci.user_agent = &ua;
+			break;
+		}
+	}
+
+	return &ci;
+}
+
+/*! \brief
+ * expects (UL_COLS - 2) columns:
+ *   contact_id, contact, expires, q, callid, cseq, flags, cflags, ua,
+ *   received, path, socket, methods, last_modified, instance, kv_store, attr)
+ */
+static inline ucontact_info_t* dbrow2info(db_val_t *vals, str *contact)
+{
+	static ucontact_info_t ci;
+	static str callid, ua, received, host, path, instance;
+	static str attr, packed_kv;
 	int port, proto;
 	char *p;
 
@@ -391,7 +294,7 @@ static inline ucontact_info_t* dbrow2info( db_val_t *vals, str *contact)
 		LM_CRIT("empty expire\n");
 		return 0;
 	}
-	ci.expires = VAL_TIME(vals+2);
+	ci.expires = VAL_INT(vals+2);
 
 	if (VAL_NULL(vals+3)) {
 		LM_CRIT("empty q\n");
@@ -420,11 +323,12 @@ static inline ucontact_info_t* dbrow2info( db_val_t *vals, str *contact)
 	ci.flags  = VAL_BITMAP(vals+6);
 
 	if (!VAL_NULL(vals+7)) {
-		flags.s   = (char *)VAL_STRING(vals+7);
+		str_const flags;
+		flags.s   = VAL_STRING(vals+7);
 		flags.len = strlen(flags.s);
 		LM_DBG("flag str: '%.*s'\n", flags.len, flags.s);
 
-		ci.cflags = flag_list_to_bitmask(&flags, FLAG_TYPE_BRANCH, FLAG_DELIM);
+		ci.cflags = flag_list_to_bitmask(&flags, FLAG_TYPE_BRANCH, FLAG_DELIM, 0);
 
 		LM_DBG("set flags: %d\n", ci.cflags);
 	}
@@ -458,7 +362,7 @@ static inline ucontact_info_t* dbrow2info( db_val_t *vals, str *contact)
 
 	/* socket name */
 	p  = (char*)VAL_STRING(vals+11);
-	if (VAL_NULL(vals+10) || p==0 || p[0]==0){
+	if (VAL_NULL(vals+11) || p==0 || p[0]==0){
 		ci.sock = 0;
 	} else {
 		if (parse_phostport( p, strlen(p), &host.s, &host.len,
@@ -493,12 +397,22 @@ static inline ucontact_info_t* dbrow2info( db_val_t *vals, str *contact)
 	}
 	ci.instance = instance;
 
-	attr.s = (char*)VAL_STRING(vals+15);
-	if (VAL_NULL(vals+15) || !attr.s) {
+	packed_kv.s = (char*)VAL_STRING(vals+15);
+	if (VAL_NULL(vals+15) || !packed_kv.s) {
+		packed_kv.s = NULL;
+		packed_kv.len = 0;
+	} else {
+		packed_kv.len  = strlen(packed_kv.s);
+	}
+	ci.packed_kv_storage = &packed_kv;
+
+	attr.s = (char*)VAL_STRING(vals+16);
+	if (VAL_NULL(vals+16) || !attr.s) {
 		attr.s = NULL;
 		attr.len = 0;
-	} else
+	} else {
 		attr.len  = strlen(attr.s);
+	}
 
 	ci.attr = &attr;
 
@@ -514,7 +428,7 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 	char uri[MAX_URI_SIZE];
 	ucontact_info_t *ci;
 	db_row_t *row;
-	db_key_t columns[18];
+	db_key_t columns[UL_COLS];
 	db_res_t* res = NULL;
 	str user, contact;
 	char* domain;
@@ -526,6 +440,8 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 	unsigned int   rlabel;
 	UNUSED(n);
 
+	char suggest_regen=0;
+
 	urecord_t* r;
 	ucontact_t* c;
 
@@ -546,8 +462,9 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 	columns[13] = &methods_col;
 	columns[14] = &last_mod_col;
 	columns[15] = &sip_instance_col;
-	columns[16] = &attr_col;
-	columns[17] = &domain_col;
+	columns[16] = &kv_store_col;
+	columns[17] = &attr_col;
+	columns[UL_COLS - 1] = &domain_col; /* "domain" always stays last */
 
 	if (ul_dbf.use_table(_c, _d->name) < 0) {
 		LM_ERR("sql use_table failed\n");
@@ -559,21 +476,21 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 #endif
 
 	if (DB_CAPABILITY(ul_dbf, DB_CAP_FETCH)) {
-		if (ul_dbf.query(_c, 0, 0, 0, columns, 0, (use_domain)?(18):(17), 0,
-		0) < 0) {
+		if (ul_dbf.query(_c, 0, 0, 0, columns, 0,
+		                 use_domain ? UL_COLS : UL_COLS - 1, 0, 0) < 0) {
 			LM_ERR("db_query (1) failed\n");
 			return -1;
 		}
 		no_rows = estimate_available_rows( 8+32+64+4+8+128+8+4+4+64
-			+32+128+16+8+8+255+32+255, 18);
+			+32+128+16+8+8+255+255+32+255, UL_COLS);
 		if (no_rows==0) no_rows = 10;
 		if(ul_dbf.fetch_result(_c, &res, no_rows)<0) {
 			LM_ERR("fetching rows failed\n");
 			return -1;
 		}
 	} else {
-		if (ul_dbf.query(_c, 0, 0, 0, columns, 0, (use_domain)?(18):(17), 0,
-		&res) < 0) {
+		if (ul_dbf.query(_c, 0, 0, 0, columns, 0,
+		                 use_domain ? UL_COLS : UL_COLS - 1, 0, &res) < 0) {
 			LM_ERR("db_query failed\n");
 			return -1;
 		}
@@ -590,6 +507,7 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 	do {
 		LM_DBG("loading records - cycle [%d]\n", ++n);
 		for(i = 0; i < RES_ROW_N(res); i++) {
+			time_t old_expires = 0;
 			row = RES_ROWS(res) + i;
 
 			user.s = (char*)VAL_STRING(ROW_VALUES(row));
@@ -608,8 +526,9 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 			}
 
 			if (use_domain) {
-				domain = (char*)VAL_STRING(ROW_VALUES(row) + 17);
-				if (VAL_NULL(ROW_VALUES(row)+17) || domain==0 || domain[0]==0){
+				domain = (char*)VAL_STRING(ROW_VALUES(row) + UL_COLS - 1);
+				if (VAL_NULL(ROW_VALUES(row) + UL_COLS - 1) || !domain ||
+				     domain[0] == '\0'){
 					LM_CRIT("empty domain record for user %.*s...skipping\n",
 							user.len, user.s);
 					continue;
@@ -625,10 +544,7 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 				}
 			}
 
-			if (unpack_indexes(ci->contact_id, &aorhash, &rlabel, &clabel)) {
-				LM_ERR("unpacking failed\n");
-				return -1;
-			}
+			unpack_indexes(ci->contact_id, &aorhash, &rlabel, &clabel);
 
 			lock_udomain(_d, &user);
 
@@ -639,37 +555,75 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 					goto error;
 				}
 
+				/* set the record label */
 				sl = r->aorhash&(_d->size-1);
-				if (_d->table[sl].next_label <= rlabel)
-					_d->table[sl].next_label = rlabel;
-				else if (_d->table[sl].next_label == 0)
-					_d->table[sl].next_label = rand();
 
-				r->label = CID_NEXT_RLABEL(_d, sl);
-				r->next_clabel = CLABEL_INC_AND_TEST(clabel);
+				if ((unsigned short)r->aorhash == aorhash) {
+					r->label = rlabel;
+				}/* else we'll get in trouble below */
+
 			} else if (ret < 0) {
 				unlock_udomain(_d, &user);
 				goto error;
 			} else {
-				if (r->next_clabel <= clabel)
-					r->next_clabel = CLABEL_INC_AND_TEST(clabel);
+				/* record found */
+				sl = r->aorhash&(_d->size-1);
 			}
 
 			if ((unsigned short)r->aorhash != aorhash) {
-				LM_ERR("failed to match aorhashes for user %.*s,"
-						"db aorhash [%u] new aorhash [%u],"
-						"db contactid [%" PRIu64 "]\n",
-						user.len, user.s, aorhash,
-						(unsigned short)(r->aorhash&(_d->size-1)),
-						ci->contact_id);
-				if (ret > 0) {
-					LM_DBG("release bogus urecord\n");
-					release_urecord(r, 0);
+				/* we've got an invalid contact;
+				 * if regeneration not set we throw error else we will try generate
+				 * new indexes for record and contact labels */
+				if ( !cid_regen ) {
+					suggest_regen=1;
+					LM_ERR("failed to match aorhashes for user %.*s,"
+							"db aorhash [%u] new aorhash [%u],"
+							"db contactid [%" PRIu64 "]\n",
+							user.len, user.s, aorhash,
+							(unsigned short)(r->aorhash&(_d->size-1)),
+							ci->contact_id);
+					if (ret > 0) {
+						LM_DBG("release bogus urecord\n");
+						release_urecord(r, 0);
+					}
+					unlock_udomain(_d, &user);
+					continue;
+				} else {
+					/* invalid contact
+					 * regenerate aor label and contact label if they're not */
+					if ( r->label == 0 ) {
+						if (_d->table[sl].next_label == 0)
+							_d->table[sl].next_label = rand();
+
+						r->label = CID_NEXT_RLABEL(_d, sl);
+					} else {
+						if (_d->table[sl].next_label == 0)
+							_d->table[sl].next_label = r->label;
+					}
+
+					if (r->next_clabel == 0)
+						r->next_clabel = rand();
+
+					old_expires = ci->expires;
+
+					/* mark contact with broken contact id as expired for deletion */
+					ci->expires = 1;
 				}
-				unlock_udomain(_d, &user);
-				continue;
+			} else {
+				/* we've got a valid contact */
+				/* update indexes accordingly */
+				sl = r->aorhash&(_d->size-1);
+
+				if (_d->table[sl].next_label <= rlabel)
+					_d->table[sl].next_label = rlabel + 1;
+
+				if (r->next_clabel <= clabel || r->next_clabel == 0)
+					r->next_clabel = CLABEL_NEXT(clabel);
+
+				r->label = rlabel;
 			}
 
+
 			if ( (c=mem_insert_ucontact(r, &contact, ci)) == 0) {
 				LM_ERR("inserting contact failed\n"
 						"Found a bad contact with id:[%" PRIu64 "] "
@@ -687,7 +641,47 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 
 			/* We have to do this, because insert_ucontact sets state to CS_NEW
 			 * and we have the contact in the database already */
-			c->state = CS_SYNC;
+			/* if contact id regeneration requested then we need to update the
+			 * database so we set the state to CS_DIRTY */
+			if ( !cid_regen )
+				c->state = CS_SYNC;
+			else {
+				/* mark for removal if we've it has an invalid aorhash */
+				if (old_expires)
+					c->state = CS_DIRTY;
+				else
+					c->state = CS_SYNC;
+			}
+
+			/* if we've found a broken contact id and regeneration set
+			 * reinsert the newly created contact that will have a valid contact id */
+			if (cid_regen && old_expires) {
+				/* rebuild the contact id for this contact */
+				ci->contact_id = pack_indexes(r->aorhash, r->label, r->next_clabel);
+				r->next_clabel = CLABEL_NEXT(r->next_clabel);
+
+				ci->expires = old_expires;
+
+				if ( (c=mem_insert_ucontact(r, &contact, ci)) == 0) {
+					LM_ERR("inserting contact failed\n"
+							"Found a bad contact with id:[%" PRIu64 "] "
+							"aor:[%.*s] contact:[%.*s] received:[%.*s]!\n"
+							"Will continue but that contact needs to be REMOVED!!\n",
+							ci->contact_id,
+							r->aor.len, r->aor.s,
+							contact.len, contact.s,
+							ci->received.len, ci->received.s);
+					unlock_udomain(_d, &user);
+					free_ucontact(c);
+					continue;
+				}
+
+				/* mark for database insertion */
+				c->state = CS_NEW;
+
+				LM_DBG("regenerated contact id to %"PRIu64"\n", ci->contact_id);
+			}
+
 			unlock_udomain(_d, &user);
 		}
 
@@ -704,6 +698,16 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 
 	ul_dbf.free_result(_c, res);
 
+	if ( suggest_regen ) {
+		LM_NOTICE("At least 1 contact(s) from the database has invalid contact_id!\n"
+				"Possible causes for this can be:\n"
+				"\t* you are migrating your location table from a version older than 2.2\n"
+				"\t* you have changed 'hash_size' module parameter from "
+				"when current contact_id's were generated;\n"
+				"If you want to regenerate new contact_id's for the broken entries"
+				" enable 'regen_broken_contactid' module parameter.\n");
+	}
+
 	/* for each not populated slot with record label
 	 * populate it*/
 	for (sl=0; sl < _d->size; sl++) {
@@ -729,7 +733,7 @@ urecord_t* db_load_urecord(db_con_t* _c, udomain_t* _d, str *_aor)
 {
 	/*static db_ps_t my_ps = NULL;*/
 	ucontact_info_t *ci;
-	db_key_t columns[16];
+	db_key_t columns[UL_COLS - 2];
 	db_key_t keys[2];
 	db_val_t vals[2];
 	db_key_t order = &q_col;
@@ -759,7 +763,8 @@ urecord_t* db_load_urecord(db_con_t* _c, udomain_t* _d, str *_aor)
 	columns[12] = &methods_col;
 	columns[13] = &last_mod_col;
 	columns[14] = &sip_instance_col;
-	columns[15] = &attr_col;
+	columns[15] = &kv_store_col;
+	columns[16] = &attr_col;
 
 	if (desc_time_order)
 		order = &last_mod_col;
@@ -788,10 +793,9 @@ urecord_t* db_load_urecord(db_con_t* _c, udomain_t* _d, str *_aor)
 		return 0;
 	}
 
-	/* CON_PS_REFERENCE(_c) = &my_ps; - this is still dangerous with STMT */
-
-	if (ul_dbf.query(_c, keys, 0, vals, columns, (use_domain)?2:1, 16, order,
-				&res) < 0) {
+	/* CON_SET_CURR_PS(_c, &my_ps); - this is still dangerous with STMT */
+	if (ul_dbf.query(_c, keys, 0, vals, columns, use_domain ? 2:1, UL_COLS - 2,
+	                 order, &res) < 0) {
 		LM_ERR("db_query failed\n");
 		return 0;
 	}
@@ -813,7 +817,6 @@ urecord_t* db_load_urecord(db_con_t* _c, udomain_t* _d, str *_aor)
 			continue;
 		}
 
-
 		if ( r==0 )
 			get_static_urecord( _d, _aor, &r);
 
@@ -833,6 +836,201 @@ urecord_t* db_load_urecord(db_con_t* _c, udomain_t* _d, str *_aor)
 	return r;
 }
 
+int
+cdb_load_urecord_locations(const udomain_t *_d, const str *_aor, urecord_t *_r)
+{
+	static const cdb_key_t aor_key = {str_init("aor"), 0}; /* TODO */
+	static const cdb_key_t home_ip_key = {str_init("home_ip"), 0}; /* TODO */
+	struct list_head *_;
+	cdb_filter_t *aor_filter;
+	int_str_t val;
+	cdb_res_t res;
+	cdb_row_t *row;
+	cdb_pair_t *pair;
+	ucontact_t *ct;
+	str my_sip_addr;
+
+	shm_free_all(_r->remote_aors);
+	_r->remote_aors = NULL;
+
+	if (clusterer_api.get_my_sip_addr(location_cluster, &my_sip_addr) != 0) {
+		LM_ERR("failed to get local PoP SIP addr\n");
+		return -1;
+	}
+
+	val.is_str = 1;
+	val.s = *_aor;
+
+	aor_filter = cdb_append_filter(NULL, &aor_key, CDB_OP_EQ, &val);
+	if (!aor_filter) {
+		LM_ERR("oom\n");
+		return -1;
+	}
+
+	LM_DBG("querying AoR %.*s\n", _aor->len, _aor->s);
+
+	if (cdbf.query(cdbc, aor_filter, &res) != 0) {
+		LM_ERR("query failed for AoR %.*s\n", _aor->len, _aor->s);
+		goto out_err;
+	}
+
+	LM_DBG("res.count: %d\n", res.count);
+
+	if (res.count == 0)
+		goto out;
+
+	list_for_each (_, &res.rows) {
+		row = list_entry(_, cdb_row_t, list);
+		pair = cdb_dict_fetch(&home_ip_key, &row->dict);
+		if (!pair) {
+			LM_ERR("metadata with no home_ip, aor: %.*s", _aor->len, _aor->s);
+			continue;
+		}
+
+		if (str_match(&my_sip_addr, &pair->val.val.st)) {
+			LM_DBG("skipping my own SIP addr (%.*s)\n",
+			       my_sip_addr.len, my_sip_addr.s);
+			continue;
+		}
+
+		ct = shm_malloc(sizeof *ct + 4 + _aor->len + 4 + pair->val.val.st.len);
+		if (!ct) {
+			LM_ERR("oom\n");
+			goto out_err;
+		}
+		memset(ct, 0, sizeof *ct);
+
+		/* future R-URI */
+		ct->c.s = (char *)(ct + 1);
+		memcpy(ct->c.s, "sip:", 4);
+		memcpy(ct->c.s + 4, _aor->s, _aor->len);
+		ct->c.len = 4 + _aor->len;
+
+		/* future outbound proxy */
+		ct->received.s = ct->c.s + ct->c.len;
+		memcpy(ct->received.s, "sip:", 4);
+		memcpy(ct->received.s + 4, pair->val.val.st.s, pair->val.val.st.len);
+		ct->received.len = 4 + pair->val.val.st.len;
+
+		ct->flags = FL_EXTRA_HOP;
+		ct->next = _r->remote_aors;
+		_r->remote_aors = ct;
+	}
+
+out:
+	cdb_free_rows(&res);
+	cdb_free_filters(aor_filter);
+	pkg_free(my_sip_addr.s);
+	return 0;
+
+out_err:
+	cdb_free_rows(&res);
+	cdb_free_filters(aor_filter);
+	shm_free_all(_r->remote_aors);
+	pkg_free(my_sip_addr.s);
+	return -1;
+}
+
+/*! \brief
+ * loads from cache DB all contacts of an AOR
+ */
+urecord_t* cdb_load_urecord(const udomain_t* _d, const str *_aor)
+{
+	static const cdb_key_t aor_key = {str_init("aor"), 1}; /* TODO */
+	struct list_head *_;
+	ucontact_info_t *ci;
+	cdb_filter_t *aor_filter;
+	int_str_t val;
+	cdb_res_t res;
+	cdb_row_t *row;
+	cdb_pair_t *contacts, *pair;
+	str contact, contacts_key = str_init("contacts"); /* TODO */
+
+	urecord_t *r;
+	ucontact_t *c;
+
+	val.is_str = 1;
+	val.s = *_aor;
+
+	aor_filter = cdb_append_filter(NULL, &aor_key, CDB_OP_EQ, &val);
+	if (!aor_filter) {
+		LM_ERR("oom\n");
+		return NULL;
+	}
+
+	LM_DBG("querying AoR %.*s\n", _aor->len, _aor->s);
+
+	if (cdbf.query(cdbc, aor_filter, &res) != 0) {
+		LM_ERR("query failed for AoR %.*s\n", _aor->len, _aor->s);
+		goto out_null;
+	}
+
+	/* TODO: implement use table _d->name */
+
+	if (res.count == 0) {
+		LM_DBG("aor %.*s not found in table %.*s\n", _aor->len, _aor->s,
+		       _d->name->len, _d->name->s);
+		cdb_free_filters(aor_filter);
+		return NULL;
+	}
+
+	if (res.count != 1)
+		LM_BUG("more than 1 result for AoR %.*s\n", _aor->len, _aor->s);
+
+	r = NULL;
+
+	row = list_entry(res.rows.next, cdb_row_t, list);
+	list_for_each (_, &row->dict) {
+		contacts = list_entry(_, cdb_pair_t, list);
+		if (str_match(&contacts->key.name, &contacts_key)) {
+			if (contacts->val.type == CDB_NULL)
+				goto done_loading;
+
+			goto have_contacts;
+		}
+	}
+
+	LM_ERR("no 'contacts' field for AoR %.*s\n", _aor->len, _aor->s);
+	goto out_null;
+
+have_contacts:
+	list_for_each (_, &contacts->val.val.dict) {
+		pair = list_entry(_, cdb_pair_t, list);
+
+		ci = cdb_ctdict2info(&pair->val.val.dict, &contact);
+		if (!ci) {
+			LM_ERR("skipping record for %.*s in table %s\n",
+			       _aor->len, _aor->s, _d->name->s);
+			continue;
+		}
+		/* save also the name of the key, to be used later, during
+		 * the update operation */
+		ci->cdb_key = pair->key.name;
+
+		if (!r)
+			get_static_urecord(_d, _aor, &r);
+
+		if (!(c = mem_insert_ucontact(r, &contact, ci))) {
+			LM_ERR("mem_insert failed\n");
+			free_urecord(r);
+			goto out_null;
+		}
+
+		/* We have to do this, because insert_ucontact sets state to CS_NEW
+		 * and we have the contact in the database already */
+		c->state = CS_SYNC;
+	}
+
+done_loading:
+	cdb_free_rows(&res);
+	cdb_free_filters(aor_filter);
+	return r;
+
+out_null:
+	cdb_free_filters(aor_filter);
+	cdb_free_rows(&res);
+	return NULL;
+}
 
 int db_timer_udomain(udomain_t* _d)
 {
@@ -841,24 +1039,24 @@ int db_timer_udomain(udomain_t* _d)
 	db_op_t  ops[2];
 	db_val_t vals[2];
 
-	if (my_ps==NULL) {
-		keys[0] = &expires_col;
-		ops[0] = "<";
-		keys[1] = &expires_col;
-		ops[1] = "!=";
+	if (ul_dbf.use_table(ul_dbh, _d->name) < 0) {
+		LM_ERR("failed to change table\n");
+		return -1;
 	}
 
 	memset(vals, 0, sizeof vals);
 
-	vals[0].type = DB_DATETIME;
-	vals[0].val.time_val = act_time + 1;
+	keys[0] = &expires_col;
+	ops[0] = "<";
+	vals[0].type = DB_INT;
+	vals[0].val.int_val = act_time + 1;
 
-	vals[1].type = DB_DATETIME;
-	vals[1].val.time_val = 0;
-
-	CON_PS_REFERENCE(ul_dbh) = &my_ps;
-	ul_dbf.use_table(ul_dbh, _d->name);
+	keys[1] = &expires_col;
+	ops[1] = "!=";
+	vals[1].type = DB_INT;
+	vals[1].val.int_val = 0;
 
+	CON_SET_CURR_PS(ul_dbh, &my_ps);
 	if (ul_dbf.delete(ul_dbh, keys, ops, vals, 2) < 0) {
 		LM_ERR("failed to delete from table %s\n",_d->name->s);
 		return -1;
@@ -919,7 +1117,7 @@ int mem_insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
 		return -1;
 	}
 
-	ul_raise_event(ei_ins_id, *_r);
+	ul_raise_aor_event(ei_ins_id, *_r);
 	update_stat( _d->users, 1);
 	return 0;
 }
@@ -930,7 +1128,7 @@ int mem_insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
  */
 void mem_delete_urecord(udomain_t* _d, struct urecord* _r)
 {
-	ul_raise_event(ei_del_id, _r);
+	ul_raise_aor_event(ei_del_id, _r);
 	slot_rem(_r->slot, _r);
 	free_urecord(_r);
 	update_stat( _d->users, -1);
@@ -975,10 +1173,20 @@ int mem_timer_udomain(udomain_t* _d)
 				flush=1;
 
 			/* Remove the entire record if it is empty */
-			if (ptr->contacts == 0)
+			if (ptr->no_clear_ref <= 0 && ptr->contacts == NULL)
 			{
+				if (exists_ulcb_type(UL_AOR_EXPIRE))
+					run_ul_callbacks(UL_AOR_EXPIRE, ptr, NULL);
+
+				if (location_cluster) {
+					if (cluster_mode == CM_FEDERATION_CACHEDB &&
+					    cdb_update_urecord_metadata(&ptr->aor, 1) != 0)
+						LM_ERR("failed to delete metadata, aor: %.*s\n",
+						       ptr->aor.len, ptr->aor.s);
+				}
+
 				iterator_delete(&prev);
-				mem_delete_urecord(_d,ptr);
+				mem_delete_urecord(_d, ptr);
 			}
 		}
 
@@ -1011,9 +1219,9 @@ int mem_timer_udomain(udomain_t* _d)
 void lock_udomain(udomain_t* _d, str* _aor)
 {
 	unsigned int sl;
-	if (db_mode!=DB_ONLY)
+	if (have_mem_storage())
 	{
-		sl = core_hash(_aor, 0, _d->size);
+		sl = core_hash(_aor, NULL, _d->size);
 
 #ifdef GEN_LOCK_T_PREFERED
 		lock_get(_d->table[sl].lock);
@@ -1030,9 +1238,9 @@ void lock_udomain(udomain_t* _d, str* _aor)
 void unlock_udomain(udomain_t* _d, str* _aor)
 {
 	unsigned int sl;
-	if (db_mode!=DB_ONLY)
+	if (have_mem_storage())
 	{
-		sl = core_hash(_aor, 0, _d->size);
+		sl = core_hash(_aor, NULL, _d->size);
 #ifdef GEN_LOCK_T_PREFERED
 		lock_release(_d->table[sl].lock);
 #else
@@ -1046,7 +1254,7 @@ void unlock_udomain(udomain_t* _d, str* _aor)
  */
 void lock_ulslot(udomain_t* _d, int i)
 {
-	if (db_mode!=DB_ONLY)
+	if (have_mem_storage())
 #ifdef GEN_LOCK_T_PREFERED
 		lock_get(_d->table[i].lock);
 #else
@@ -1060,7 +1268,7 @@ void lock_ulslot(udomain_t* _d, int i)
  */
 void unlock_ulslot(udomain_t* _d, int i)
 {
-	if (db_mode!=DB_ONLY)
+	if (have_mem_storage())
 #ifdef GEN_LOCK_T_PREFERED
 		lock_release(_d->table[i].lock);
 #else
@@ -1069,69 +1277,201 @@ void unlock_ulslot(udomain_t* _d, int i)
 }
 
 
+int cdb_update_urecord_metadata(const str *_aor, int unpublish)
+{
+	static const cdb_key_t id_key = {str_init("id"), 1}; /* TODO */
+	static str id_print_buf;
+	cdb_filter_t *id_filter = NULL;
+	int_str_t val;
+	cdb_dict_t my_pop_info;
+	str sip_addr;
+
+	LM_DBG("%spublishing metadata for AoR %.*s\n", unpublish ? "un" : "",
+	       _aor->len, _aor->s);
+
+	cdb_dict_init(&my_pop_info);
+
+	if (clusterer_api.get_my_sip_addr(location_cluster, &sip_addr) != 0) {
+		LM_ERR("failed to get local PoP SIP addr\n");
+		return -1;
+	}
+
+	if (pkg_str_extend(&id_print_buf, _aor->len + sip_addr.len) != 0) {
+		LM_ERR("oom\n");
+		goto out_err;
+	}
+
+	memcpy(id_print_buf.s, _aor->s, _aor->len);
+	memcpy(id_print_buf.s + _aor->len, sip_addr.s, sip_addr.len);
+
+	val.is_str = 1;
+	val.s.s = id_print_buf.s;
+	val.s.len = _aor->len + sip_addr.len;
+
+	if (unpublish) {
+		if (cdbf._remove(cdbc, &val.s, &id_key.name) < 0) {
+			LM_ERR("fail to del metadata, AoR %.*s\n", _aor->len, _aor->s);
+			return -1;
+		}
+
+		goto out;
+	}
+
+	id_filter = cdb_append_filter(NULL, &id_key, CDB_OP_EQ, &val);
+	if (!id_filter) {
+		LM_ERR("oom\n");
+		goto out_err;
+	}
+
+	if (CDB_DICT_ADD_STR(&my_pop_info, "aor", _aor) != 0 ||
+	    CDB_DICT_ADD_STR(&my_pop_info, "home_ip", &sip_addr) != 0) {
+		goto out_err;
+	}
+
+	dbg_cdb_dict("my pop: ", &my_pop_info);
+
+	if (cdbf.update(cdbc, id_filter, &my_pop_info) < 0) {
+		LM_ERR("cache update query for AoR %.*s failed!\n",
+		       _aor->len, _aor->s);
+		goto out_err;
+	}
+
+out:
+	pkg_free(sip_addr.s);
+	cdb_free_filters(id_filter);
+	cdb_free_entries(&my_pop_info, NULL);
+	return 0;
+
+out_err:
+	pkg_free(sip_addr.s);
+	cdb_free_filters(id_filter);
+	cdb_free_entries(&my_pop_info, NULL);
+	return -1;
+}
+
 
 /*! \brief
  * Create and insert a new record
- * after inserting and urecord one must populate the
- * label field outside this function
  */
 int insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r,
-                   char is_replicated)
+                   char skip_replication)
 {
-	int sl;
-
-	if (db_mode!=DB_ONLY) {
+	if (have_mem_storage()) {
 		if (mem_insert_urecord(_d, _aor, _r) < 0) {
 			LM_ERR("inserting record failed\n");
 			return -1;
 		}
-		/* make sure it does not overflows 14 bits */
-		(*_r)->next_clabel = (rand()&CLABEL_MASK);
-		sl = (*_r)->aorhash&(_d->size-1);
 
-		(*_r)->label = CID_NEXT_RLABEL(_d, sl);
+		if (!skip_replication) {
+			init_urecord_labels(*_r, _d);
 
-		if (!is_replicated && ul_replicate_cluster)
-			replicate_urecord_insert(*_r);
+			if (cluster_mode == CM_FEDERATION_CACHEDB
+			        && cdb_update_urecord_metadata(_aor, 0) != 0) {
+				LM_ERR("failed to publish cachedb location for AoR %.*s\n",
+				       _aor->len, _aor->s);
+			}
+
+			if (location_cluster)
+				replicate_urecord_insert(*_r);
+		}
 	} else {
 		get_static_urecord( _d, _aor, _r);
 	}
 
+	if (exists_ulcb_type(UL_AOR_INSERT))
+		run_ul_callbacks(UL_AOR_INSERT, *_r, NULL);
+
 	return 0;
 }
 
+static inline urecord_t *find_mem_urecord(udomain_t *_d, const str *_aor)
+{
+	unsigned int sl, aorhash;
+	urecord_t **r;
+
+	aorhash = core_hash(_aor, NULL, 0);
+	sl = aorhash & (_d->size - 1);
+
+	r = (urecord_t **)map_find(_d->table[sl].records, *_aor);
+	return r ? *r : NULL;
+}
 
 /*! \brief
  * obtain urecord pointer if urecord exists;
  */
 int get_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
 {
-	unsigned int sl, aorhash;
 	urecord_t* r;
-	void ** dest;
 
-	if (db_mode!=DB_ONLY) {
-		/* search in cache */
-		aorhash = core_hash(_aor, 0, 0);
-		sl = aorhash&(_d->size-1);
-
-		dest = map_find(_d->table[sl].records, *_aor);
-
-		if( dest == NULL )
-			return 1;
-
-		*_r = *dest;
+	switch (cluster_mode) {
+	case CM_NONE:
+	case CM_FULL_SHARING:
+	case CM_FEDERATION_CACHEDB:
+		r = find_mem_urecord(_d, _aor);
+		if (!r)
+			goto out;
 
+		*_r = r;
 		return 0;
-	} else {
+	case CM_FULL_SHARING_CACHEDB:
+		r = cdb_load_urecord(_d, _aor);
+		if (r) {
+			*_r = r;
+			return 0;
+		}
+		break;
+	case CM_SQL_ONLY:
 		/* search in DB */
 		r = db_load_urecord( ul_dbh, _d, _aor);
 		if (r) {
 			*_r = r;
 			return 0;
 		}
+		break;
+	default:
+		abort();
 	}
 
+out:
+	*_r = NULL;
+	return 1;   /* Nothing found */
+}
+
+/*! \brief
+ * Only relevant in a federation @cluster_mode.
+ * Obtain urecord pointer if AoR exists in at least one location.
+ *
+ * This function performs two lookups:
+ *  - mem lookup, thus providing @_r->contacts
+ *  - cachedb query, populating @_r->remote_aors
+ */
+int get_global_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
+{
+	urecord_t* r;
+
+	switch (cluster_mode) {
+	case CM_FEDERATION_CACHEDB:
+		r = find_mem_urecord(_d, _aor);
+		if (!r)
+			get_static_urecord(_d, _aor, &r);
+
+		if (cdb_load_urecord_locations(_d, _aor, r) != 0) {
+			if (r->is_static)
+				goto out;
+		}
+
+		/* static, empty record -> return "not found" instead */
+		if (r->is_static && !r->remote_aors)
+			goto out;
+
+		*_r = r;
+		return 0;
+	default:
+		abort();
+	}
+
+out:
+	*_r = NULL;
 	return 1;   /* Nothing found */
 }
 
@@ -1139,40 +1479,64 @@ int get_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
  * Delete a urecord from domain
  */
 int delete_urecord(udomain_t* _d, str* _aor, struct urecord* _r,
-                   char is_replicated)
+                   char skip_replication)
 {
 	struct ucontact* c, *t;
 
-	if (db_mode==DB_ONLY) {
-		if (_r==0)
-			get_static_urecord( _d, _aor, &_r);
-		if (db_delete_urecord(_r)<0) {
+	switch (cluster_mode) {
+	case CM_SQL_ONLY:
+		if (!_r)
+			get_static_urecord(_d, _aor, &_r);
+		if (db_delete_urecord(_r) < 0) {
 			LM_ERR("DB delete failed\n");
 			return -1;
 		}
 		free_urecord(_r);
 		return 0;
+
+	case CM_FULL_SHARING_CACHEDB:
+		if (!_r)
+			get_static_urecord(_d, _aor, &_r);
+		if (cdb_delete_urecord(_r) < 0) {
+			LM_ERR("failed to delete %.*s from cache\n", _aor->len, _aor->s);
+			return -1;
+		}
+		free_urecord(_r);
+		return 0;
+
+	case CM_FEDERATION_CACHEDB:
+		if (!skip_replication && cdb_update_urecord_metadata(_aor, 1) != 0)
+			LM_ERR("failed to delete metadata, aor: %.*s\n",
+			       _aor->len, _aor->s);
+		break;
+
+	default:
+		break;
 	}
 
-	if (_r==0) {
+	if (!_r) {
 		if (get_urecord(_d, _aor, &_r) > 0) {
 			return 0;
 		}
 	}
 
-	if (!is_replicated && ul_replicate_cluster)
-		replicate_urecord_delete(_r);
-
 	c = _r->contacts;
 	while(c) {
 		t = c;
 		c = c->next;
-		if (delete_ucontact(_r, t, is_replicated) < 0) {
+		if (delete_ucontact(_r, t, NULL, skip_replication) < 0) {
 			LM_ERR("deleting contact failed\n");
 			return -1;
 		}
 	}
-	release_urecord(_r, is_replicated);
+
+	if (_r->no_clear_ref > 0)
+		return 0;
+
+	if (!skip_replication && location_cluster)
+		replicate_urecord_delete(_r);
+
+	release_urecord(_r, skip_replication);
 	return 0;
 }
 
