@@ -1064,6 +1064,8 @@ static inline void tcpconn_destroy(struct tcp_connection* tcpconn)
 		/* force timeout */
 		tcpconn->lifetime=0;
 		tcpconn->state=S_CONN_BAD;
+		sh_log(tcpconn->hist, TCP_DEL_DELAY, "tcpconn_destroy delayed, (%d)",
+			tcpconn->refcnt);
 		LM_DBG("delaying (%p, flags %04x) ref = %d ...\n",
 				tcpconn, tcpconn->flags, tcpconn->refcnt);
 
@@ -1373,6 +1375,9 @@ inline static int handle_tcp_worker(struct tcp_worker* tcp_c, int fd_i)
 			tcpconn->flags&=~F_CONN_REMOVED_WRITE;
 			break;
 		case CONN_ERROR_TCPW:
+			LM_ERR("TCP_DBG - main: received conn %p / %u as faulty "
+				"(state %d, rfcnt=%d)\n", tcpconn, tcpconn->id,
+				tcpconn->state, tcpconn->refcnt);
 		case CONN_DESTROY:
 		case CONN_EOF:
 			/* WARNING: this will auto-dec. refcnt! */
@@ -1526,8 +1531,11 @@ inline static int handle_worker(struct process_table* p, int fd_i)
 			tcpconn->flags&=~F_CONN_REMOVED_WRITE;
 			break;
 		case ASYNC_WRITE_GENW:
+			sh_log(tcpconn->hist,TCP_UNREF,"ASYNC_WRITE_GENW, (%d)",
+				tcpconn->refcnt);
 			if (tcpconn->state==S_CONN_BAD){
 				tcpconn->lifetime=0;
+				tcpconn_put(tcpconn);
 				break;
 			}
 			tcpconn_put(tcpconn);
@@ -2277,6 +2285,7 @@ mi_response_t *mi_tcp_list_conns(const mi_params_t *params,
 					/* add one node for each conn */
 					add_mi_number( conn_item, MI_SSTR("Alias port"),
 						conn->con_aliases[j].port );
+
 			}
 		}
 
