@@ -29,10 +29,12 @@
 
 
 #include "../../ut.h"
+#include "../../trim.h"
 #include "../../mem/mem.h"
 #include "../../mem/shm_mem.h"
 #include "../parser_f.h"
 #include "../parse_content.h"
+#include "../parse_body.h"
 #include "sdp.h"
 #include "sdp_helpr_funcs.h"
 
@@ -45,26 +47,25 @@
 /**
  * Creates and initialize a new sdp_info structure
  */
-static inline int new_sdp(struct sip_msg* _m)
+static inline sdp_info_t* new_sdp(void)
 {
 	sdp_info_t* sdp;
 
 	sdp = (sdp_info_t*)pkg_malloc(sizeof(sdp_info_t));
 	if (sdp == NULL) {
 		LM_ERR("No memory left\n");
-		return -1;
+		return NULL;
 	}
 	memset( sdp, 0, sizeof(sdp_info_t));
 
-	_m->sdp = sdp;
-
-	return 0;
+	return sdp;
 }
 
+
 /**
  * Alocate a new session cell.
  */
-static inline sdp_session_cell_t *add_sdp_session(sdp_info_t* _sdp, int session_num, str* cnt_disp)
+static inline sdp_session_cell_t *add_sdp_session(sdp_info_t* _sdp, int session_num, str* cnt_disp, str body)
 {
 	sdp_session_cell_t *session;
 	int len;
@@ -83,6 +84,8 @@ static inline sdp_session_cell_t *add_sdp_session(sdp_info_t* _sdp, int session_
 		session->cnt_disp.len = cnt_disp->len;
 	}
 
+	session->body = body;
+
 	/* Insert the new session */
 	session->next = _sdp->sessions;
 	_sdp->sessions = session;
@@ -95,7 +98,7 @@ static inline sdp_session_cell_t *add_sdp_session(sdp_info_t* _sdp, int session_
  * Allocate a new stream cell.
  */
 static inline sdp_stream_cell_t *add_sdp_stream(sdp_session_cell_t* _session, int stream_num,
-		str* media, str* port, str* transport, str* payloads, int is_rtp, int pf, str* sdp_ip)
+		str* media, str* port, str* transport, str* payloads, int is_rtp, int pf, str* sdp_ip, str body)
 {
 	sdp_stream_cell_t *stream;
 	int len;
@@ -125,6 +128,8 @@ static inline sdp_stream_cell_t *add_sdp_stream(sdp_session_cell_t* _session, in
 	stream->ip_addr.s = sdp_ip->s;
 	stream->ip_addr.len = sdp_ip->len;
 
+	stream->body = body;
+
 	/* Insert the new stream */
 	stream->next = _session->streams;
 	_session->streams = stream;
@@ -207,10 +212,6 @@ static inline sdp_payload_attr_t** init_p_payload_attr(sdp_stream_cell_t* _strea
 
 void set_sdp_payload_attr(sdp_payload_attr_t *payload_attr, str *rtp_enc, str *rtp_clock, str *rtp_params)
 {
-	if (payload_attr == NULL) {
-		LM_ERR("Invalid payload location\n");
-		return;
-	}
 	payload_attr->rtp_enc.s = rtp_enc->s;
 	payload_attr->rtp_enc.len = rtp_enc->len;
 	payload_attr->rtp_clock.s = rtp_clock->s;
@@ -227,28 +228,77 @@ void set_sdp_payload_fmtp(sdp_payload_attr_t *payload_attr, str *fmtp_string )
 		LM_ERR("Invalid payload location\n");
 		return;
 	}
+
 	payload_attr->fmtp_string.s = fmtp_string->s;
 	payload_attr->fmtp_string.len = fmtp_string->len;
 
 	return;
 }
 
+void try_set_fmtp_payload_link(sdp_stream_cell_t *stream,sdp_payload_attr_t *payload_attr, str *fmtp_string )
+{
+	str link_str=str_init("apt=");
+	char *p_start,*p_end;
+	str ref_payload_s;
+	sdp_payload_attr_t *ref_payload;
+
+	if (payload_attr == NULL) {
+		LM_ERR("Invalid payload location\n");
+		return;
+	}
+
+	/* apt=xxx */
+	if (fmtp_string->len < 4) {
+		LM_DBG("Too small body \n");
+		return;
+	}
+
+	LM_DBG("We need to link [%.*s] to [%.*s]\n",fmtp_string->len,fmtp_string->s,payload_attr->rtp_payload.len,payload_attr->rtp_payload.s);
+
+	if ((p_start = str_strstr(fmtp_string,&link_str)) == NULL) {
+		return;
+	}
+
+	/* skip it */
+	p_start += link_str.len; 
+	ref_payload_s.s = p_start;
+	for (p_end=p_start;p_end<fmtp_string->s+fmtp_string->len;p_end++) {
+		if (*p_end==';')
+			break;
+	}
+
+	ref_payload_s.len = p_end-p_start;
+	trim(&ref_payload_s);
+
+	ref_payload = (sdp_payload_attr_t*)get_sdp_payload4payload(stream, &ref_payload_s);
+	if (ref_payload == NULL)
+		return;
+
+	LM_DBG("Found it, %.*s going to %.*s\n",ref_payload_s.len,ref_payload_s.s,ref_payload->rtp_payload.len,ref_payload->rtp_payload.s);
+
+	ref_payload->linked_payload = payload_attr; 
+	return;
+}
+
+void set_sdp_payload_custom_attr(sdp_payload_attr_t *payload_attr, str *attr )
+{
+	if (payload_attr == NULL) {
+		LM_DBG("No payload_attr\n");
+		return;
+	}
+	if (payload_attr->custom_attrs_size == MAX_CUSTOM_ATTRS-1) {
+		LM_DBG("Max custom a= attrs reached \n");
+		return;
+	}
+
+	payload_attr->custom_attrs[payload_attr->custom_attrs_size++] = *attr;
+	return;
+}
+
 /*
  * Getters ....
  */
-int get_sdp_session_num(struct sip_msg* _m)
-{
-	if (_m->sdp == NULL) return 0;
-	return _m->sdp->sessions_num;
-}
-
-int get_sdp_stream_num(struct sip_msg* _m)
-{
-	if (_m->sdp == NULL) return 0;
-	return _m->sdp->streams_num;
-}
-
-sdp_session_cell_t* get_sdp_session_sdp(struct sdp_info* sdp, int session_num)
+sdp_session_cell_t* get_sdp_session(sdp_info_t* sdp, int session_num)
 {
 	sdp_session_cell_t *session;
 
@@ -261,14 +311,8 @@ sdp_session_cell_t* get_sdp_session_sdp(struct sdp_info* sdp, int session_num)
 	return NULL;
 }
 
-sdp_session_cell_t* get_sdp_session(struct sip_msg* _m, int session_num)
-{
-	if (_m->sdp == NULL) return NULL;
-	return get_sdp_session_sdp(_m->sdp, session_num);
-}
-
-
-sdp_stream_cell_t* get_sdp_stream_sdp(struct sdp_info* sdp, int session_num, int stream_num)
+sdp_stream_cell_t* get_sdp_stream(sdp_info_t* sdp, int session_num,
+																int stream_num)
 {
 	sdp_session_cell_t *session;
 	sdp_stream_cell_t *stream;
@@ -293,14 +337,6 @@ sdp_stream_cell_t* get_sdp_stream_sdp(struct sdp_info* sdp, int session_num, int
 	return NULL;
 }
 
-sdp_stream_cell_t* get_sdp_stream(struct sip_msg* _m, int session_num, int stream_num)
-{
-	if (_m->sdp == NULL) return NULL;
-	return get_sdp_stream_sdp(_m->sdp, session_num, stream_num);
-
-}
-
-
 sdp_payload_attr_t* get_sdp_payload4payload(sdp_stream_cell_t *stream, str *rtp_payload)
 {
 	sdp_payload_attr_t *payload;
@@ -344,6 +380,45 @@ sdp_payload_attr_t* get_sdp_payload4index(sdp_stream_cell_t *stream, int index)
 	return stream->p_payload_attr[index];
 }
 
+/**
+ * Add SDP attribute.
+ */
+static inline int add_sdp_attr(str line, sdp_attr_t **list)
+{
+	char *p;
+	static sdp_attr_t *last;
+	sdp_attr_t *a = pkg_malloc(sizeof *a);
+	if (!a) {
+		LM_ERR("No memory left!\n");
+		return -1;
+	}
+	memset(a, 0, sizeof *a);
+
+	/* check for value separator : */
+	p = q_memchr(line.s + 2, ':', line.len - 2);
+	if (p) {
+		a->attribute.s = line.s + 2;
+		a->attribute.len = p - a->attribute.s;
+		/* skip the separator */
+		a->value.s = p + 1;
+		a->value.len = line.len - (a->value.s - line.s);
+		trim(&a->value);
+	} else {
+		a->attribute.s = line.s + 2;
+		a->attribute.len = line.len - 2;
+	}
+	trim(&a->attribute);
+
+	/* link it to the list */
+	if (*list)
+		last->next = a;
+	else
+		*list = a;
+	/* we remember the last object added */
+	last = a;
+
+	return 0;
+}
 
 /**
  * SDP parser method.
@@ -358,7 +433,7 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 	int is_rtp;
 	char *bodylimit;
 	char *v1p, *o1p, *m1p, *m2p, *c1p, *c2p, *a1p, *a2p, *b1p;
-	str tmpstr1;
+	str tmpstr1,custom_attr;
 	int stream_num, payloadnum, pf;
 	sdp_session_cell_t *session;
 	sdp_stream_cell_t *stream;
@@ -378,9 +453,22 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 	 * unconditionally.
 	 */
 	bodylimit = body.s + body.len;
-	v1p = find_sdp_line(body.s, bodylimit, 'v');
-	if (v1p == NULL) {
-		LM_ERR("no sessions in SDP\n");
+	if (body.len < 2) {
+		LM_ERR("body too short!\n");
+		return -1;
+	}
+	v1p = body.s;
+	/* skip over the first initial \r\n  (optional) */
+	if (*v1p == '\r' || *v1p == '\n')
+		v1p++;
+	if (*v1p == '\r' || *v1p == '\n')
+		v1p++;
+	if (v1p + 1 >= bodylimit) {
+		LM_ERR("body too short %ld!\n", (long)(bodylimit - v1p));
+		return -1;
+	}
+	if (*v1p != 'v' || *(v1p+1) != '=') {
+		LM_ERR("invalid SDP start: [%c%c]\n", *v1p, *(v1p + 1));
 		return -1;
 	}
 	/* get session origin */
@@ -396,7 +484,7 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 		return -1;
 	}
 	/* Allocate a session cell */
-	session = add_sdp_session(_sdp, session_num, cnt_disp);
+	session = add_sdp_session(_sdp, session_num, cnt_disp, body);
 	if (session == NULL) return -1;
 
 	/* Get origin IP */
@@ -430,6 +518,22 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 		extract_bwidth(&tmpstr1, &session->bw_type, &session->bw_width);
 	}
 
+	/* from b1p or o1p down to m1p we have session a= attributes */
+	a1p = find_sdp_line((b1p?b1p:o1p), m1p, 'a');
+	a2p = a1p;
+	for (;;) {
+		a1p = a2p;
+		if (a1p == NULL || a1p >= m1p)
+			break;
+		a2p = find_next_sdp_line(a1p + 1, m1p, 'a', m1p);
+
+		tmpstr1.s = a1p;
+		tmpstr1.len = a2p - a1p;
+
+		if (add_sdp_attr(tmpstr1, &session->attr) < 0)
+			return -1;
+	}
+
 	/* Have session. Iterate media descriptions in session */
 	m2p = m1p;
 	stream_num = 0;
@@ -455,6 +559,8 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 				LM_ERR("can't find media IP in the message\n");
 				return -1;
 			}
+			/* take the family from the more specific line */
+			pf = session->pf;
 		}
 
 		/* Extract the port on sdp_port */
@@ -466,7 +572,7 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 		}
 
 		/* Allocate a stream cell */
-		stream = add_sdp_stream(session, stream_num, &sdp_media, &sdp_port, &sdp_transport, &sdp_payload, is_rtp, pf, &sdp_ip);
+		stream = add_sdp_stream(session, stream_num, &sdp_media, &sdp_port, &sdp_transport, &sdp_payload, is_rtp, pf, &sdp_ip, tmpstr1);
 		if (stream == 0) return -1;
 
 		/* increment total number of streams */
@@ -533,13 +639,17 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 					a1p = rtp_clock.s + rtp_clock.len;
 				}
 				payload_attr = (sdp_payload_attr_t*)get_sdp_payload4payload(stream, &rtp_payload);
-				set_sdp_payload_attr(payload_attr, &rtp_enc, &rtp_clock, &rtp_params);
+				if (payload_attr)
+					set_sdp_payload_attr(payload_attr, &rtp_enc, &rtp_clock, &rtp_params);
 			} else if (extract_rtcp(&tmpstr1, &stream->rtcp_port) == 0) {
 				a1p = stream->rtcp_port.s + stream->rtcp_port.len;
 			} else if (parse_payload_attr && extract_fmtp(&tmpstr1,&rtp_payload,&fmtp_string) == 0){
+				LM_DBG("Found FMTP [%.*s] for payload [%.*s]\n",fmtp_string.len,fmtp_string.s,rtp_payload.len,rtp_payload.s);
 				a1p = fmtp_string.s + fmtp_string.len;
 				payload_attr = (sdp_payload_attr_t*)get_sdp_payload4payload(stream, &rtp_payload);
 				set_sdp_payload_fmtp(payload_attr, &fmtp_string);
+
+				try_set_fmtp_payload_link(stream,payload_attr,&fmtp_string);
 			} else if (extract_accept_types(&tmpstr1, &stream->accept_types) == 0) {
 				a1p = stream->accept_types.s + stream->accept_types.len;
 			} else if (extract_accept_wrapped_types(&tmpstr1, &stream->accept_wrapped_types) == 0) {
@@ -548,11 +658,23 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 				a1p = stream->max_size.s + stream->max_size.len;
 			} else if (extract_path(&tmpstr1, &stream->path) == 0) {
 				a1p = stream->path.s + stream->path.len;
-			/*} else { */
-			/*	LM_DBG("else: `%.*s'\n", tmpstr1.len, tmpstr1.s); */
+			} else {
+				if (parse_payload_attr && extract_custom_a_attr(&tmpstr1,&rtp_payload,&custom_attr) == 0) {
+					LM_DBG("extracted attr [%.*s] for payload [%.*s]\n",custom_attr.len,custom_attr.s,rtp_payload.len,rtp_payload.s);
+					a1p = custom_attr.s + custom_attr.len;
+					payload_attr = (sdp_payload_attr_t*)get_sdp_payload4payload(stream, &rtp_payload);
+					set_sdp_payload_custom_attr(payload_attr, &custom_attr);
+				} else {
+					LM_DBG("else: parse_payload_attr ? %d `%.*s'\n",parse_payload_attr, tmpstr1.len, tmpstr1.s);
+				}
 			}
 
+			tmpstr1.s = a2p;
 			a2p = find_next_sdp_line(a1p-1, m2p, 'a', m2p);
+			tmpstr1.len = a2p - tmpstr1.s;
+
+			if (add_sdp_attr(tmpstr1, &stream->attr) < 0)
+				return -1;
 		}
 		/* Let's detect if the media is on hold by checking
 		 * the good old "0.0.0.0" connection address */
@@ -560,11 +682,11 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 			if (stream->ip_addr.s && stream->ip_addr.len) {
 				if (stream->ip_addr.len == HOLD_IP_LEN &&
 					strncmp(stream->ip_addr.s, HOLD_IP_STR, HOLD_IP_LEN)==0)
-					stream->is_on_hold = 1;
+					stream->is_on_hold = RFC2543_HOLD;
 			} else if (session->ip_addr.s && session->ip_addr.len) {
 				if (session->ip_addr.len == HOLD_IP_LEN &&
 					strncmp(session->ip_addr.s, HOLD_IP_STR, HOLD_IP_LEN)==0)
-					stream->is_on_hold = 1;
+					stream->is_on_hold = RFC2543_HOLD;
 			}
 		}
 		++stream_num;
@@ -572,193 +694,65 @@ int parse_sdp_session(str *sdp_body, int session_num, str *cnt_disp, sdp_info_t*
 	return 0;
 }
 
-static int parse_mixed_content(str *mixed_body, str delimiter, sdp_info_t* _sdp)
-{
-	int res, no_eoh_found, start_parsing;
-	char *bodylimit, *rest;
-	char *d1p, *d2p;
-	char *ret, *end;
-	unsigned int mime;
-	str sdp_body, cnt_disp;
-	int session_num;
-	struct hdr_field hf;
-
-	bodylimit = mixed_body->s + mixed_body->len;
-	d1p = find_sdp_line_delimiter(mixed_body->s, bodylimit, delimiter);
-	if (d1p == NULL) {
-		LM_ERR("empty multipart content\n");
-		return -1;
-	}
-	d2p = d1p;
-	session_num = 0;
-	for(;;) {
-		/* Per-application iteration */
-		d1p = d2p;
-		if (d1p == NULL || d1p >= bodylimit)
-			break; /* No applications left */
-		d2p = find_next_sdp_line_delimiter(d1p, bodylimit, delimiter, bodylimit);
-		/* d2p is text limit for application parsing */
-		memset(&hf,0, sizeof(struct hdr_field));
-		rest = eat_line(d1p + delimiter.len + 2, d2p - d1p - delimiter.len - 2);
-		if ( rest > d2p ) {
-			LM_ERR("Unparsable <%.*s>\n", (int)(d2p-d1p), d1p);
-			return -1;
-		}
-		no_eoh_found = 1;
-		start_parsing = 0;
-		/*LM_DBG("we need to parse this: <%.*s>\n", d2p-rest, rest); */
-		while( rest<d2p && no_eoh_found ) {
-			rest = get_sdp_hdr_field(rest, d2p, &hf);
-			switch (hf.type){
-			case HDR_EOH_T:
-				no_eoh_found = 0;
-				break;
-			case HDR_CONTENTTYPE_T:
-				end = hf.body.s + hf.body.len;
-				ret = decode_mime_type(hf.body.s, end , &mime, NULL);
-				if (ret==0)
-					return -1;
-				if (ret!=end) {
-					LM_ERR("the header CONTENT_TYPE contains "
-						"more then one mime type :-(!\n");
-					return -1;
-				}
-				if ((mime&0x00ff)==SUBTYPE_ALL || (mime>>16)==TYPE_ALL) {
-					LM_ERR("invalid mime with wildcard '*' in Content-Type hdr!\n");
-					return -1;
-				}
-			    	//LM_DBG("HDR_CONTENTTYPE_T: %d:%d %p-> <%.*s:%.*s>\n",mime&0x00ff,mime>>16,
-				//			hf.name.s,hf.name.len,hf.name.s,hf.body.len,hf.body.s);
-				if (((((unsigned int)mime)>>16) == TYPE_APPLICATION) && ((mime&0x00ff) == SUBTYPE_SDP)) {
-			    		/*LM_DBG("start_parsing: %d:%d\n",mime&0x00ff,mime>>16); */
-					start_parsing = 1;
-				}
-				break;
-			case HDR_CONTENTDISPOSITION_T:
-				cnt_disp.s = hf.body.s;
-				cnt_disp.len = hf.body.len;
-				break;
-			case HDR_ERROR_T:
-				return -1;
-				break;
-			default:
-				LM_DBG("unknown header: <%.*s:%.*s>\n",hf.name.len,hf.name.s,hf.body.len,hf.body.s);
-			}
-		} /* end of while */
-		/* and now we need to parse the content */
-		if (start_parsing) {
-			sdp_body.s = rest;
-			sdp_body.len = d2p-rest;
-			/* LM_DBG("we need to check session %d: <%.*s>\n", session_num, sdp_body.len, sdp_body.s); */
-			res = parse_sdp_session(&sdp_body, session_num, &cnt_disp, _sdp);
-			if (res != 0) {
-				/* _sdp is freed by the calling function */
-				return -1;
-			}
-			session_num++;
-		}
-	}
-	return 0;
-}
 
 /**
- * Parse SDP.
+ * Parse all SDP parts from the SIP body.
  *
- * returns 0 on success.
- * non zero on error.
+ * returns the first found SDP on success
+ * returns NULL if no SDP found or if error
  */
-int parse_sdp(struct sip_msg* _m)
+sdp_info_t* parse_sdp(struct sip_msg* _m)
 {
-	int res;
-	str body, mp_delimiter;
-	int mime;
+	struct body_part *part;
+	sdp_info_t *sdp, *ret;
 
-	if (_m->sdp) {
-		return 0;  /* Already parsed */
-	}
-
-	if (get_body(_m, &body)!=0 || body.len==0) {
+	if ( parse_sip_body(_m)<0 || _m->body==NULL) {
 		LM_DBG("message body has length zero\n");
-		return 1;
+		return NULL;
 	}
 
-	mime = parse_content_type_hdr(_m);
-	if (mime <= 0) {
-		return -1;
-	}
-	switch (((unsigned int)mime)>>16) {
-	case TYPE_APPLICATION:
-		/* LM_DBG("TYPE_APPLICATION: %d\n",((unsigned int)mime)>>16); */
-		switch (mime&0x00ff) {
-		case SUBTYPE_SDP:
-			/* LM_DBG("SUBTYPE_SDP: %d\n",mime&0x00ff); */
-			if (new_sdp(_m) < 0) {
-				LM_ERR("Can't create sdp\n");
-				return -1;
-			}
-			res = parse_sdp_session(&body, 0, NULL, _m->sdp);
-			if (res != 0) {
-				LM_DBG("free_sdp\n");
-				free_sdp((sdp_info_t**)(void*)&(_m->sdp));
-			}
-			return res;
-			break;
-		default:
-			LM_DBG("TYPE_APPLICATION: unknown %d\n",mime&0x00ff);
-			return -1;
+	ret = NULL;
+
+	/* iterate all body parts and look for the SDP mime */
+	for( part=&_m->body->first ; part ; part=part->next) {
+
+		/* skip body parts which were deleted or newly added */
+		if (!is_body_part_received(part))
+			continue;
+
+		if ( part->mime != ((TYPE_APPLICATION<<16)+SUBTYPE_SDP) )
+			continue;
+
+		if (part->parsed) {
+			if (!ret)
+				ret = part->parsed;
+			continue;
 		}
-		break;
-	case TYPE_MULTIPART:
-		/* LM_DBG("TYPE_MULTIPART: %d\n",((unsigned int)mime)>>16); */
-		switch (mime&0x00ff) {
-		case SUBTYPE_MIXED:
-			/* LM_DBG("SUBTYPE_MIXED: %d <%.*s>\n",mime&0x00ff,_m->content_type->body.len,_m->content_type->body.s); */
-			if(get_mixed_part_delimiter(&(_m->content_type->body),&mp_delimiter) > 0) {
-				/*LM_DBG("got delimiter: <%.*s>\n",mp_delimiter.len,mp_delimiter.s); */
-				if (new_sdp(_m) < 0) {
-					LM_ERR("Can't create sdp\n");
-					return -1;
-				}
-				res = parse_mixed_content(&body, mp_delimiter, _m->sdp);
-				if (res != 0) {
-					LM_DBG("free_sdp\n");
-					free_sdp((sdp_info_t**)(void*)&(_m->sdp));
-				}
-				return res;
+
+		if ( (sdp=new_sdp())==NULL ) {
+			LM_ERR("Can't create new sdp, skipping\n");
+		} else {
+			if (parse_sdp_session(&part->body, 0, NULL, sdp)<0) {
+				LM_ERR("failed to parse SDP for body part, skipping\n");
+				free_sdp( sdp );
 			} else {
-				return -1;
+				part->parsed = (void*)sdp;
+				part->free_parsed_f = (free_parsed_part_function)free_sdp;
+				/* remember the first found SDP */
+				if (!ret) ret = sdp;
 			}
-			break;
-		default:
-			LM_DBG("TYPE_MULTIPART: unknown %d\n",mime&0x00ff);
-			return -1;
 		}
-		break;
-	default:
-		LM_DBG("%d\n",((unsigned int)mime)>>16);
-		return -1;
 	}
 
-	LM_CRIT("We should not see this!\n");
-	return res;
+	return ret;
 }
 
-
-/**
- * Free all memory.
- */
-void free_sdp(sdp_info_t** sdp)
-{
-	__free_sdp(*sdp);
-	pkg_free(*sdp);
-	*sdp = NULL;
-}
-
-void __free_sdp(sdp_info_t* sdp)
+void free_sdp_content(sdp_info_t* sdp)
 {
 	sdp_session_cell_t *session, *l_session;
 	sdp_stream_cell_t *stream, *l_stream;
 	sdp_payload_attr_t *payload, *l_payload;
+	sdp_attr_t *attr, *l_attr;
 
 	LM_DBG("sdp = %p\n", sdp);
 	if (sdp == NULL) return;
@@ -768,6 +762,12 @@ void __free_sdp(sdp_info_t* sdp)
 	while (session) {
 		l_session = session;
 		session = session->next;
+		attr = l_session->attr;
+		while (attr) {
+			l_attr = attr;
+			attr = attr->next;
+			pkg_free(l_attr);
+		}
 		stream = l_session->streams;
 		while (stream) {
 			l_stream = stream;
@@ -778,6 +778,12 @@ void __free_sdp(sdp_info_t* sdp)
 				payload = payload->next;
 				pkg_free(l_payload);
 			}
+			attr = l_stream->attr;
+			while (attr) {
+				l_attr = attr;
+				attr = attr->next;
+				pkg_free(l_attr);
+			}
 			if (l_stream->p_payload_attr) {
 				pkg_free(l_stream->p_payload_attr);
 			}
@@ -1196,9 +1202,9 @@ error:
 	return NULL;
 }
 
-sdp_info_t * clone_sdp_info(struct sip_msg* _m)
+sdp_info_t * clone_sdp_info(sdp_info_t *sdp_info)
 {
-	sdp_info_t *clone_sdp_info, *sdp_info=_m->sdp;
+	sdp_info_t *clone_sdp_info;
 	sdp_session_cell_t *clone_session, *prev_clone_session, *session;
 	int i, len;
 
