@@ -49,15 +49,27 @@
 #include <sys/sockio.h>
 #endif
 
+#include "str.h"
 #include "globals.h"
 #include "socket_info.h"
 #include "dprint.h"
 #include "mem/mem.h"
 #include "ut.h"
+#include "pt_scaling.h"
 #include "resolve.h"
 #include "name_alias.h"
 #include "net/trans.h"
 
+#ifdef __OS_linux
+#include <features.h>     /* for GLIBC version testing */
+#if defined(__GLIBC_PREREQ)
+#if __GLIBC_PREREQ(2, 4)
+#include <ifaddrs.h>
+#define HAVE_IFADDRS
+#endif
+#endif
+#endif
+
 #define MAX_PROC_BUFFER	256
 
 /* list manip. functions (internal use only) */
@@ -101,10 +113,7 @@
 
 
 /* another helper function, it just creates a socket_info struct */
-struct socket_info* new_sock_info(	char* name,
-								unsigned short port, unsigned short proto,
-								char *adv_name, unsigned short adv_port,
-								unsigned short children,enum si_flags flags)
+static struct socket_info* new_sock_info( struct socket_id *sid)
 {
 	struct socket_info* si;
 
@@ -112,38 +121,76 @@ struct socket_info* new_sock_info(	char* name,
 	if (si==0) goto error;
 	memset(si, 0, sizeof(struct socket_info));
 	si->socket=-1;
-	si->name.len=strlen(name);
-	si->name.s=(char*)pkg_malloc(si->name.len+1); /* include \0 */
-	if (si->name.s==0) goto error;
-	memcpy(si->name.s, name, si->name.len+1);
-	/* set port & proto */
-	si->port_no=port;
-	si->proto=proto;
-	si->flags=flags;
 
-    /* advertised socket information */
+	if (sid->name) {
+		si->name.len=strlen(sid->name);
+		si->name.s=(char*)pkg_malloc(si->name.len+1); /* include \0 */
+		if (si->name.s==0) goto error;
+		memcpy(si->name.s, sid->name, si->name.len+1);
+	}
+
+	/* set port & proto */
+	si->port_no=sid->port;
+	si->proto=sid->proto;
+	si->flags=sid->flags;
+
+	/* advertised socket information */
 	/* Make sure the adv_sock_string is initialized, because if there is
 	 * no adv_sock_name, no other code will initialize it!
 	 */
 	si->adv_sock_str.s=NULL;
 	si->adv_sock_str.len=0;
-	if(adv_name) {
-		si->adv_name_str.len=strlen(adv_name);
+	si->adv_port = 0; /* Here to help grep_sock_info along. */
+	if(sid->adv_name) {
+		si->adv_name_str.len=strlen(sid->adv_name);
 		si->adv_name_str.s=(char *)pkg_malloc(si->adv_name_str.len+1);
 		if (si->adv_name_str.s==0) goto error;
-		memcpy(si->adv_name_str.s, adv_name, si->adv_name_str.len+1);
-	}
-	si->adv_port = 0; /* Here to help grep_sock_info along. */
-	if(adv_port) {
+		memcpy(si->adv_name_str.s, sid->adv_name, si->adv_name_str.len+1);
+		if (!sid->adv_port) sid->adv_port=si->port_no ;
 		si->adv_port_str.s=pkg_malloc(10);
 		if (si->adv_port_str.s==0) goto error;
-		si->adv_port_str.len=snprintf(si->adv_port_str.s, 10, "%hu", adv_port);
-		si->adv_port = adv_port;
+		si->adv_port_str.len=snprintf(si->adv_port_str.s, 10, "%hu",
+			(unsigned short)sid->adv_port);
+		si->adv_port = sid->adv_port;
 	}
-	if ( (si->proto==PROTO_TCP || si->proto==PROTO_TLS) && children) {
-		LM_WARN("number of children per TCP/TLS listener not supported -> ignoring...\n");
+
+	/* store the tag info too */
+	if (sid->tag) {
+		si->tag.len = strlen(sid->tag);
+		si->tag.s=(char*)pkg_malloc(si->tag.len+1); /* include \0 */
+		if (si->tag.s==0) goto error;
+		memcpy(si->tag.s, sid->tag, si->tag.len+1);
+	}
+
+	if (si->proto!=PROTO_UDP && si->proto!=PROTO_SCTP &&
+	        si->proto!=PROTO_HEP_UDP) {
+		if (sid->workers)
+			LM_WARN("number of workers per non UDP-based <%.*s> listener not "
+				"supported -> ignoring...\n", si->name.len, si->name.s);
+		if (sid->auto_scaling_profile)
+			LM_WARN("auto-scaling for non UDP-based <%.*s> listener not "
+				"supported -> ignoring...\n", si->name.len, si->name.s);
 	} else {
-		si->children = children;
+		if (sid->workers)
+			si->workers = sid->workers;
+		if (sid->auto_scaling_profile) {
+			si->s_profile = get_scaling_profile(sid->auto_scaling_profile);
+			if (si->s_profile==NULL) {
+				LM_WARN("scaling profile <%s> in listener <%.*s> not defined "
+					"-> ignoring it...\n", sid->auto_scaling_profile,
+					si->name.len, si->name.s);
+			} else {
+				auto_scaling_enabled = 1;
+			}
+		} else if (udp_auto_scaling_profile) {
+			si->s_profile = get_scaling_profile(udp_auto_scaling_profile);
+			if (si->s_profile==NULL) {
+				LM_WARN("scaling profile <%s> in udp_workers not defined "
+					"-> ignoring it...\n", udp_auto_scaling_profile);
+			} else {
+				auto_scaling_enabled = 1;
+			}
+		}
 	}
 	return si;
 error:
@@ -159,10 +206,14 @@ static void free_sock_info(struct socket_info* si)
 {
 	if(si){
 		if(si->name.s) pkg_free(si->name.s);
+		if(si->tag.s) pkg_free(si->tag.s);
+		if(si->sock_str.s) pkg_free(si->sock_str.s);
 		if(si->address_str.s) pkg_free(si->address_str.s);
 		if(si->port_no_str.s) pkg_free(si->port_no_str.s);
 		if(si->adv_name_str.s) pkg_free(si->adv_name_str.s);
 		if(si->adv_port_str.s) pkg_free(si->adv_port_str.s);
+		if(si->adv_sock_str.s) pkg_free(si->adv_sock_str.s);
+		if(si->tag_sock_str.s) pkg_free(si->tag_sock_str.s);
 	}
 }
 
@@ -175,8 +226,8 @@ static void free_sock_info(struct socket_info* si)
  * WARNING: uses str2ip6 so it will overwrite any previous
  *  unsaved result of this function (static buffer)
  */
-struct socket_info* grep_sock_info(str* host, unsigned short port,
-												unsigned short proto)
+struct socket_info* grep_sock_info_ext(str* host, unsigned short port,
+										unsigned short proto, int check_tags)
 {
 	char* hname;
 	int h_len;
@@ -211,10 +262,17 @@ struct socket_info* grep_sock_info(str* host, unsigned short port,
 						h_len, hname,
 						si->name.len, si->name.s
 				);
+
+			if (check_tags && port==0 && si->tag.s && h_len==si->tag.len &&
+			strncasecmp(hname, si->tag.s, si->tag.len)==0 )
+				goto found;
+
 			if (port) {
 				LM_DBG("checking if port %d matches port %d\n",
 						si->port_no, port);
-				if (si->port_no!=port && si->adv_port!=port) {
+				if ((si->port_no != 0 || protos[c_proto].default_port != port) &&
+				    (si->port_no == 0 || si->port_no != port) &&
+				    si->adv_port!=port) {
 					continue;
 				}
 			}
@@ -233,12 +291,27 @@ struct socket_info* grep_sock_info(str* host, unsigned short port,
 				* can be in mixed case, it will also match
 				* ipv6 addresses if we are lucky*/
 				goto found;
+			/* if no advertised is specified on the interface, we should check
+			 * if it is the global address */
+			if (!si->adv_name_str.len && default_global_address->s &&
+				h_len == default_global_address->len &&
+				(strncasecmp(hname, default_global_address->s,
+					default_global_address->len)==0) /*slower*/)
+				/* this might match sockets that are not supposed to
+				 * match, when using multiple listeners for the same
+				 * protocol; but in that case the default_global_address
+				 * concept is broken, since there is no way to choose
+				 * the right socket */
+				goto found;
 			/* check if host == ip address */
 			/* ipv6 case is uglier, host can be [3ffe::1] */
 			ip6=str2ip6(host);
 			if (ip6){
 				if (ip_addr_cmp(ip6, &si->address))
 					goto found; /* match */
+				else
+				if (si->adv_name_str.len && ip_addr_cmp(ip6,&si->adv_address))
+					goto found;
 				else
 					continue; /* no match, but this is an ipv6 address
 								 so no point in trying ipv4 */
@@ -302,16 +375,33 @@ found:
 }
 
 
+/* parses the specified `spec` and returns an associated
+ * socket_info*, if it could be found */
+struct socket_info* parse_sock_info(str *addr)
+{
+	int port, proto;
+	str host;
+
+	if (!addr || !addr->s)
+		return NULL;
+
+	if (parse_phostport(addr->s, addr->len, &host.s, &host.len,
+		&port, &proto) != 0) {
+		return NULL;
+	}
+
+	return grep_internal_sock_info(&host, (unsigned short) port,
+		(unsigned short) proto);
+}
+
 
 /* adds a new sock_info structure to the corresponding list
  * return  0 on success, -1 on error */
-int new_sock2list(char* name, unsigned short port, unsigned short proto,
-		char *adv_name, unsigned short adv_port, unsigned short children,
-		enum si_flags flags, struct socket_info** list)
+int new_sock2list(struct socket_id *sid, struct socket_info** list)
 {
 	struct socket_info* si;
 
-	si=new_sock_info(name, port, proto, adv_name, adv_port, children, flags);
+	si=new_sock_info(sid);
 	if (si==0){
 		LM_ERR("new_sock_info failed\n");
 		goto error;
@@ -324,47 +414,66 @@ error:
 
 
 
-/* adds a sock_info structure to the corresponding proto list
- * return  0 on success, -1 on error */
-int add_listen_iface(char* name, unsigned short port, unsigned short proto,
-			char *adv_name, unsigned short adv_port, unsigned short children,
-			enum si_flags flags)
-{
-	struct socket_info** list;
-	unsigned short c_proto;
-
-	c_proto=(proto)?proto:PROTO_UDP;
-	LM_INFO("XXX - c_proto = %d\n",c_proto);
-	do{
-		list=get_sock_info_list(c_proto);
-		if (list==0){
-			LM_ERR("get_sock_info_list failed\n");
-			goto error;
-		}
-		if (port==0) /* use default port */
-			port=protos[c_proto].default_port;
-		if (new_sock2list(name, port, c_proto, adv_name, adv_port, children,
-		flags, list)<0){
-			LM_ERR("new_sock2list failed\n");
-			goto error;
-		}
-	}while( (proto==0) && (c_proto=next_proto(c_proto)));
-	return 0;
-error:
-	return -1;
-}
-
-
-
 /* add all family type addresses of interface if_name to the socket_info array
- * if if_name==0, adds all addresses on all interfaces
  * WARNING: it only works with ipv6 addresses on FreeBSD
  * return: -1 on error, 0 on success
  */
-int add_interfaces(char* if_name, int family, unsigned short port,
-					unsigned short proto, unsigned short children,
-					struct socket_info** list)
+int expand_interface(struct socket_info *si, struct socket_info** list)
 {
+	int ret = -1;
+	struct ip_addr addr;
+	struct socket_id sid;
+
+	sid.port = si->port_no;
+	sid.proto = si->proto;
+	sid.workers = si->workers;
+	sid.auto_scaling_profile = si->s_profile?si->s_profile->name:NULL;
+	sid.adv_port = si->adv_port;
+	sid.adv_name = si->adv_name_str.s; /* it is NULL terminated */
+	sid.tag = si->tag.s; /* it is NULL terminated */
+#ifdef HAVE_IFADDRS
+	/* use the getifaddrs interface to get all the interfaces */
+	struct ifaddrs *addrs;
+	struct ifaddrs *it;
+
+	if (getifaddrs(&addrs) != 0) {
+		LM_ERR("cannot get interfaces list: %s(%d)\n", strerror(errno), errno);
+		return -1;
+	}
+
+	for (it = addrs; it; it = it->ifa_next) {
+		if (!it->ifa_addr)
+			continue;
+
+		if (si->name.len == 0 || (strcmp(si->name.s, it->ifa_name) == 0)) {
+			if (it->ifa_addr->sa_family != AF_INET &&
+					it->ifa_addr->sa_family != AF_INET6)
+				continue;
+			/*
+			 * if it is ipv6, and there was no explicit interface specified,
+			 * make sure we don't add any "scoped" interface
+			 */
+			if (it->ifa_addr->sa_family == AF_INET6 &&
+					(((struct sockaddr_in6 *)(void *)it->ifa_addr)->sin6_scope_id != 0))
+
+				continue;
+			sockaddr2ip_addr(&addr, it->ifa_addr);
+			if ((sid.name = ip_addr2a(&addr)) == 0)
+				goto end;
+			sid.flags = si->flags;
+			if (it->ifa_flags & IFF_LOOPBACK)
+				sid.flags |= SI_IS_LO;
+			if (new_sock2list(&sid, list) != 0) {
+				LM_ERR("clone_sock2list failed\n");
+				goto end;
+			}
+			ret = 0;
+		}
+	}
+end:
+	freeifaddrs(addrs);
+	return ret;
+#else
 	struct ifconf ifc;
 	struct ifreq ifr;
 	struct ifreq ifrcopy;
@@ -373,10 +482,6 @@ int add_interfaces(char* if_name, int family, unsigned short port,
 	int size;
 	int lastlen;
 	int s;
-	char* tmp;
-	struct ip_addr addr;
-	int ret;
-	enum si_flags flags;
 
 #ifdef HAVE_SOCKADDR_SA_LEN
 	#ifndef MAX
@@ -384,9 +489,7 @@ int add_interfaces(char* if_name, int family, unsigned short port,
 	#endif
 #endif
 	/* ipv4 or ipv6 only*/
-	flags=SI_NONE;
-	s=socket(family, SOCK_DGRAM, 0);
-	ret=-1;
+	s=socket(AF_INET, SOCK_DGRAM, 0);
 	lastlen=0;
 	ifc.ifc_req=0;
 	for (size=100; ; size*=2){
@@ -430,7 +533,7 @@ int add_interfaces(char* if_name, int family, unsigned short port,
 		/* copy contents into ifr structure
 		 * warning: it might be longer (e.g. ipv6 address) */
 		memcpy(&ifr, p, sizeof(ifr));
-		if (ifr.ifr_addr.sa_family!=family){
+		if (ifr.ifr_addr.sa_family!=AF_INET){
 			/*printf("strange family %d skipping...\n",
 					ifr->ifr_addr.sa_family);*/
 			continue;
@@ -440,27 +543,26 @@ int add_interfaces(char* if_name, int family, unsigned short port,
 		ifrcopy=ifr;
 		if (ioctl(s, SIOCGIFFLAGS,  &ifrcopy)!=-1){ /* ignore errors */
 			/* ignore down ifs only if listening on all of them*/
-			if (if_name==0){
+			if (si->name.len==0){
 				/* if if not up, skip it*/
 				if (!(ifrcopy.ifr_flags & IFF_UP)) continue;
 			}
 		}
 
-
-
-		if ((if_name==0)||
-			(strncmp(if_name, ifr.ifr_name, sizeof(ifr.ifr_name))==0)){
+		if (si->name.len == 0 ||
+			strncmp(si->name.s, ifr.ifr_name, sizeof(ifr.ifr_name))==0){
 
 			/*add address*/
 			sockaddr2ip_addr(&addr,
 					(struct sockaddr*)(p+(long)&((struct ifreq*)0)->ifr_addr));
-			if ((tmp=ip_addr2a(&addr))==0) goto error;
+			if ((sid.name=ip_addr2a(&addr))==0) goto error;
+			sid.flags = si->flags;
 			/* check if loopback */
 			if (ifrcopy.ifr_flags & IFF_LOOPBACK)
-				flags|=SI_IS_LO;
+				sid.flags|=SI_IS_LO;
 			/* add it to one of the lists */
-			if (new_sock2list(tmp, port, proto, 0, 0, children, flags, list)!=0){
-				LM_ERR("new_sock2list failed\n");
+			if (new_sock2list(&sid, list) != 0) {
+				LM_ERR("clone_sock2list failed\n");
 				goto error;
 			}
 			ret=0;
@@ -481,9 +583,12 @@ error:
 	if (s >= 0)
 		close(s);
 	return -1;
+#endif
 }
 
 
+#define STR_IMATCH(str, buf) ((str).len==strlen(buf) && strncasecmp(buf, (str).s, (str).len)==0)
+
 /* fixes a socket list => resolve addresses,
  * interface names, fills missing members, remove duplicates */
 int fix_socket_list(struct socket_info **list)
@@ -501,8 +606,14 @@ int fix_socket_list(struct socket_info **list)
 
 	for (si=*list;si;){
 		next=si->next;
-		if (add_interfaces(si->name.s, AF_INET, si->port_no,
-							si->proto, si->children, list)!=-1){
+		// fix the SI_IS_LO flag for sockets specified by IP/hostname as expand_interface
+		// below will only do it for sockets specified using the network interface name
+		if (STR_IMATCH(si->name, "localhost") ||
+			STR_IMATCH(si->name, "127.0.0.1") ||
+			STR_IMATCH(si->name, "0:0:0:0:0:0:0:1") || STR_IMATCH(si->name, "::1")) {
+			si->flags |= SI_IS_LO;
+		}
+		if (expand_interface(si, list)!=-1){
 			/* success => remove current entry (shift the entire array)*/
 			sock_listrm(list, si);
 			free_sock_info(si);
@@ -515,8 +626,8 @@ int fix_socket_list(struct socket_info **list)
 #endif
 	for (si=*list;si;si=si->next){
 		/* fix the number of processes per interface */
-		if (!si->children && is_udp_based_proto(si->proto))
-			si->children = children_no;
+		if (!si->workers && is_udp_based_proto(si->proto))
+			si->workers = udp_workers_no;
 		if (si->port_no==0)
 			si->port_no= protos[si->proto].default_port;
 
@@ -530,7 +641,7 @@ int fix_socket_list(struct socket_info **list)
 			LM_ERR("out of pkg memory.\n");
 			goto error;
 		}
-		strncpy(si->port_no_str.s, tmp, len+1);
+		memcpy(si->port_no_str.s, tmp, len+1);
 		si->port_no_str.len=len;
 		/* get "official hostnames", all the aliases etc. */
 		he=resolvehost(si->name.s,0);
@@ -552,7 +663,7 @@ int fix_socket_list(struct socket_info **list)
 				goto error;
 			}
 			si->name.len=strlen(he->h_name);
-			strncpy(si->name.s, he->h_name, si->name.len+1);
+			memcpy(si->name.s, he->h_name, si->name.len+1);
 		}
 		/* add the aliases*/
 		if (auto_aliases) {
@@ -571,7 +682,7 @@ int fix_socket_list(struct socket_info **list)
 				goto error;
 			}
 			si->address_str.s[0] = '[';
-			strncpy( si->address_str.s+1 , tmp, strlen(tmp));
+			memcpy( si->address_str.s+1 , tmp, strlen(tmp));
 			si->address_str.s[1+strlen(tmp)] = ']';
 			si->address_str.s[2+strlen(tmp)] = '\0';
 			si->address_str.len=strlen(tmp) + 2;
@@ -581,7 +692,7 @@ int fix_socket_list(struct socket_info **list)
 				LM_ERR("out of pkg memory.\n");
 				goto error;
 			}
-			strncpy(si->address_str.s, tmp, strlen(tmp)+1);
+			memcpy(si->address_str.s, tmp, strlen(tmp)+1);
 			si->address_str.len=strlen(tmp);
 		}
 		/* set is_ip (1 if name is an ip address, 0 otherwise) */
@@ -629,6 +740,24 @@ int fix_socket_list(struct socket_info **list)
 			}
 			hostent2ip_addr(&si->adv_address, he, 0); /*convert to ip_addr */
 
+			if (si->adv_address.af == AF_INET6 /* translates to IPv6 */
+			&& str2ip6(&si->adv_name_str)!=NULL /* it's an actual IP */
+			&& si->adv_name_str.s[0]!='['  )    /* not enclosed */
+			{
+				tmp = pkg_malloc( si->adv_name_str.len +2 );
+				if (tmp==NULL) {
+					LM_ERR("failed to convert advertized IPv6 "
+						"to enclosed format\n");
+					goto error;
+				}
+				tmp[0] = '[';
+				memcpy( tmp+1, si->adv_name_str.s, si->adv_name_str.len);
+				tmp[si->adv_name_str.len+1] = ']';
+				pkg_free(si->adv_name_str.s);
+				si->adv_name_str.s = tmp;
+				si->adv_name_str.len += 2;
+			}
+
 			/* build and set string encoding for the adv socket info
 			 * This is usefful for the usrloc module when it's generating
 			 * or updating the socket on a location record, so we'll generate
@@ -649,6 +778,21 @@ int fix_socket_list(struct socket_info **list)
 			memcpy(si->adv_sock_str.s, tmp, si->adv_sock_str.len);
 		}
 
+		if (si->tag.len) {
+			/* build and set string encoding for the tagged socket info */
+			tmp = socket2str( si, 0, &si->tag_sock_str.len, 2);
+			if (tmp==0) {
+				LM_ERR("failed to convert tag socket to string\n");
+				goto error;
+			}
+			si->tag_sock_str.s=(char*)pkg_malloc(si->tag_sock_str.len);
+			if (si->tag_sock_str.s==0) {
+				LM_ERR("out of pkg memory.\n");
+				goto error;
+			}
+			memcpy(si->tag_sock_str.s, tmp, si->tag_sock_str.len);
+		}
+
 		/* build and set string encoding for the real socket info */
 		tmp = socket2str( si, 0, &si->sock_str.len, 0);
 		if (tmp==0) {
@@ -672,9 +816,10 @@ int fix_socket_list(struct socket_info **list)
 #endif /* USE_MCAST */
 
 #ifdef EXTRA_DEBUG
-		printf("              %.*s [%s]:%s%s\n", si->name.len,
+		printf("              %.*s [%s]:%s%s%s\n", si->name.len,
 				si->name.s, si->address_str.s, si->port_no_str.s,
-		                si->flags & SI_IS_MCAST ? " mcast" : "");
+		                si->flags & SI_IS_MCAST ? " mcast" : "",
+		                is_anycast(si) ? " anycast" : "");
 #endif
 	}
 	/* removing duplicate addresses*/
@@ -696,7 +841,8 @@ int fix_socket_list(struct socket_info **list)
 						(l->name.len!=si->name.len)||
 						(strncmp(l->name.s, si->name.s, si->name.len)!=0))
 					)
-					add_alias(l->name.s, l->name.len, l->port_no, l->proto);
+					if (add_alias(l->name.s,l->name.len,l->port_no,l->proto)<0)
+						LM_ERR(" add_alias failed\n");
 
 				/* remove l*/
 				sock_listrm(list, l);
@@ -766,7 +912,7 @@ error:
  *       Therefore it is CRUCIAL that you free ipList when you are done with
  *       its contents, to avoid a nasty memory leak.
  */
-int get_socket_list_from_proto(int **ipList, int protocol) {
+int get_socket_list_from_proto(unsigned int **ipList, int protocol) {
 
 	struct socket_info  *si;
 	struct socket_info** list;
@@ -805,7 +951,8 @@ int get_socket_list_from_proto(int **ipList, int protocol) {
 		return 0;
 	}
 
-	*ipList = pkg_malloc(numberOfSockets * (num_ip_octets + 1) * sizeof(int));
+	*ipList = pkg_malloc(numberOfSockets *
+			(num_ip_octets + 1) * (int)sizeof(int));
 
 	/* We couldn't allocate memory for the IP List.  So all we can do is
 	 * fail. */
@@ -853,17 +1000,17 @@ int get_socket_list_from_proto(int **ipList, int protocol) {
  * get_socket_list_from_proto() in this file.
  *
  */
-static int parse_proc_net_line(char *line, int *ipAddress, int *rx_queue)
+static int parse_proc_net_line(char *line, unsigned int *ipAddress, int *rx_queue)
 {
 	int i;
 
-	int ipOctetExtractionMask = 0xFF;
+	unsigned int ipOctetExtractionMask = 0xFF;
 
 	char *currColonLocation;
 	char *nextNonNumericalChar;
 	char *currentLocationInLine = line;
 
-	int parsedInteger[4];
+	unsigned int parsedInteger[4];
 
 	/* Example line from /proc/net/tcp or /proc/net/udp:
 	 *
@@ -936,7 +1083,7 @@ static int parse_proc_net_line(char *line, int *ipAddress, int *rx_queue)
  * get_socket_list_from_proto() in this file.
  *
  * */
-static int match_ip_and_port(int *ipOne, int *ipArray, int sizeOf_ipArray)
+static int match_ip_and_port(unsigned int *ipOne, unsigned int *ipArray, int sizeOf_ipArray)
 {
 	int curIPAddrIdx;
 	int curOctetIdx;
@@ -984,13 +1131,13 @@ static int match_ip_and_port(int *ipOne, int *ipArray, int sizeOf_ipArray)
  *       interface.  On other systems, zero will always be returned.
  */
 static int get_used_waiting_queue(
-		int forTCP, int *interfaceList, int listSize)
+		int forTCP, unsigned int *interfaceList, int listSize)
 {
 	FILE *fp;
 	char *fileToOpen;
 
 	char lineBuffer[MAX_PROC_BUFFER];
-	int  ipAddress[NUM_IP_OCTETS+1];
+	unsigned int  ipAddress[NUM_IP_OCTETS+1];
 	int  rx_queue;
 	int  waitingQueueSize = 0;
 
@@ -1046,9 +1193,9 @@ static int get_used_waiting_queue(
  */
 int get_total_bytes_waiting(int only_proto)
 {
-	static int *UDPList  = NULL;
-	static int *TCPList  = NULL;
-	static int *TLSList  = NULL;
+	static unsigned int *UDPList  = NULL;
+	static unsigned int *TCPList  = NULL;
+	static unsigned int *TLSList  = NULL;
 
 	static int numUDPSockets  = -1;
 	static int numTCPSockets  = -1;
@@ -1152,7 +1299,7 @@ int probe_max_sock_buff(int sock,int buff_choice,int buff_max,int buff_increment
 			if (phase==1) break;
 			else { phase=1; optval >>=1; continue; }
 		}
-		LM_DBG("trying : %d\n", optval );
+		/* LM_DBG("trying : %d\n", optval ); */
 		if (setsockopt( sock, SOL_SOCKET, buff_opt,
 			(void*)&optval, sizeof(optval)) ==-1){
 			/* Solaris returns -1 if asked size too big; Linux ignores */
@@ -1173,10 +1320,10 @@ int probe_max_sock_buff(int sock,int buff_choice,int buff_max,int buff_increment
 			LM_ERR("getsockopt: %s\n", strerror(errno));
 			return -1;
 		} else {
-			LM_DBG("setting %s: set=%d,verify=%d\n",info,
-				optval, voptval);
+			/*LM_DBG("setting %s: set=%d,verify=%d\n",info,
+				optval, voptval);*/
 			if (voptval<optval) {
-				LM_DBG("setting buf has no effect\n");
+				LM_DBG("setting %s buf to %d had no effect\n",info,optval);
 				/* if setting buffer size failed and still in the aggressive
 				phase, try less aggressively; otherwise give up */
 				if (phase==0) { phase=1; optval >>=1 ; continue; }
@@ -1192,7 +1339,7 @@ int probe_max_sock_buff(int sock,int buff_choice,int buff_max,int buff_increment
 		LM_ERR("getsockopt: %s\n", strerror(errno));
 		return -1;
 	}
-	LM_INFO("using %s buffer of %d kb\n",info, (foptval/1024));
+	LM_DBG("using %s buffer of %d kb\n",info, (foptval/1024));
 
 	return 0;
 }
